﻿/* Inst-ETB14up.ahk ------ UTF-8 ------------------------------------------------------------------------
              Install the required files to the appropriate locations
Comment       Normally compiled using the compiler by running !_Make-distribution
              PrepareInstallation is assumed to be on desktop together with project installation-directory
Tasks         • From here we distribute to the various target locations
              • FM information is set up by script, not here
AHK version   1.1.26.00
              Assure that AutoHotkeySC.bin is the same as  Unicode 32-bit.bin
              Only this way the compiled scripts run on both x64 and x32 systems
Copyleft      2018 Klaus Daube, Zürich, Switzerland, klaus@daube.ch
History       2010-02-29 v1.0
              2020-03-20 v9.0 See Developer-log "ETB-13up" later became *ETB-14up*
              2022-10-19 ETB v11 have only one installation for de/enfr independly of FM-version
                         PrepareInstallation will determine FMlang and prepare the files in %appdata%
                         Install-ETB14up will distribute to %appdata%
*/ ; // =========================================================================================
#Requires AutoHotkey v1.1
#SingleInstance Force
if !A_IsAdmin ; must be first in script (see H:\Utilities\DDDutils\Test-ahks\Run-as-admin.ahk)
  {           ; see *verb in AHK help --- here the context-menu Run as Administrator
  try Run *RunAs "%A_ScriptFullPath%"
  catch
    MsgBox, You clicked Cancel when asked to elevate the script`nPlease Run this again with admin rights.
  ExitApp
  } ; ------------------------------------------------------------------------
L0_SetUp:
  SetWorkingDir, %A_ScriptDir%                              ; files are in subdirs
Global  appLang                                             ; defined by System (PrepareInsallation)
      , appName     := "ETB14up"                            ; EnhancedTollBars for FM ≥ 14
      , appDate
      , appDescr                                            ; from cfg [lang]
      , appIconPath := A_ScriptDir . "\FM-puzzle.ico"       ; different location in Pre/Install
      , appInf
      , appPgm      :=   A_ScriptName                       ; for shortcut
      , appStitle
      , appTitle                                            ; from cfg [lang]
      , appVersion  := "11.0"                               ; harmonisation of languages
      , auxDir
      , bOption1    := 0                                    ; per default
      , btnCancel, btnInstall, btnUnInst, btnDownLoad, btnHelp, btnContinue
      , cfgFile     := A_ScriptDir . "\config.cfg"          ; different location in Pre/Install
      , ETBmenuData                                         ; appended to FM menu file
      , customuiData                                        ; modified version of ETBmenuData
      , customUILoc                                         ; replacement loc of $HOME\...\customUI.cfg
      , dddDir      := A_AppData . "\D+DD"
      , docDir      := A_ScriptDir . "\Docu"
      , etbdata
      , FMappDir    := A_AppData . "\Adobe\FrameMaker\"
      , FMlang                                              ; by PrepareInstallation
      , FMroot                                              ; by PrepareInstallation
      , FMtarget                                            ; by PrepareInstallation
      , helpFile                                            ; from cfg [lang]
      , installDir                                          ; project Dir in %appdata%
      , instHint                                            ; from cfg [lang]
      , option1                                             ; from cfg [lang]
      , problem     := ""
      , preDescr                                            ; from cfg [lang]
      , preTitle                                            ; from cfg [lang]
      , shortCutDir := A_StartMenu . "\D+DD\"               ; User start menu
      , srcDir                                              ; by PrepareInstallation
      , targetDir                                           ; intermed target in %appdata%
      , targetAppdata
      , targetScripts
      , userIni                                             ; maker.ini in user area
      , UpdateOK
      , URL                                                 ; URL of ancillary doc
      , util1Icon   := "TextHelper.ico"
      , util1Name   := "TextHelper"
      , util1Title  := "Utility TextHelper"
      , WebFile
      , workspaces_dir
;     , xmpFile     := "FMgraph-examples"                   ; special in FMGraph, FMbiblio

L1_Initialise:
  GetCfgData()                                              ; read from E:\_DDDprojects\..\Install..\config.ini
  If (installDir <> A_ScriptDir) {
    MsgSomethingWrong()                                     ; safety anchor
    ExitApp
  }
  FileGetTime, appDate, %A_ScriptFullPath%, M               ; Modification time
  FormatTime,  appDate, %appDate%, yyyy-MM-dd HH:mm

  etbdata   := installDir . "\ETB-" . FMlang
  appTitle  := appTitle . " for FM-" . FMtarget . ",  " . FMlang
  appStitle := appTitle
  appInf    := appVersion   . " / " . appDate               ; file date of this script/exe

  FileEncoding, UTF-8                                       ; FM-14… cfg files can be UTF-8 or Windows cp

L2_SetUpTargets:
  targetAppdata   := FMappDir . FMtarget
  workspaces_dir  := targetAppdata . "\Workspaces"
  userIni         := targetAppdata . "\maker.ini"
  targetScripts   := targetAppdata . "\Startup"           ; scripts must define a command
;MsgBox, Debugging`n scritptdir = %A_ScriptDir%`n workingdir= %A_WorkingDir%

L3_UserInterface:
Menu, Tray, Icon, %appIcon%
Gui +OwnDialogs
Gui, Font, s10, Verdana
Gui, Add, Text,     x270 y003 w200 h15,                     %appInf%
Gui, Add, Text,     x015 y025 w430 h35,                     %instHint%%FMtarget%
Gui, Add, Checkbox, x015 y080 w400 h50  vbOption1 Checked , %option1%
Gui, Add, Button,   x015 y140 w120 h025 Default gMenuInstallation,  %btnInstall%
Gui, Add, Button,   x165 y140 w120 h25  gMenuReload,        %btnDownload%
Gui, Add, Button,   x315 y140 w120 h25  gMenuUnInstall, %btnUnInst%
Gui, Add, Button,   x015 y170 w120 h25  gMenuHelp,          %btnHelp%
Gui, Add, Button,   x315 y170 w120 h25  gMenuCancel,        %btnCancel%

Gui, Show,          x800 y300 h210 w450,                    %appTitle%
Return

GuiClose:
MenuCancel:
ExitApp

MenuHelp:
  Run, http://www.daube.ch/docu/%helpFile%                  ;
Return

MenuInstallation:
  Gui, Submit                                               ; get input data
;MsgBox, Debugging`n All is ok for Installation
    Progress, CBlime A, %A_Space%, Installation progress, %appTitle%  ; A: do not stay on top
  EditCfgFile()
    Progress, 10
  EditMenu()
    Progress, 25
  if (bOption1 <> 0) {
    DownLoadDocs()
    Progress, 50
; MsgBox Check downlaoded files
  }
  EditETBextras()                    ; Script ETBextras
    Progress, 60
  InstallSpecialDirs()
    Progress, 80
  EditIniFile()
  CreateShortCut()
  Finalise1()
ExitApp ; --- end MenuInstallation

MenuUnInstall:                          ; === Remove the files from the User area =================
  FileSetAttrib, -R, %targetAppdata%\*.*, 1, 1             ; also folders, recurse
  If (ErrorLevel > 0)
    MsgATTRIBfailure()
  FileRemoveDir, %targetAppdata%\etb_icons, 1              ; Dir inc files

  FileRemoveDir, %targetScripts%\ETBextras, 1              ; Dir inc files
  FileDelete,    %targetScripts%\ETB*.*                    ; Files only
; MsgBox, Deleted scripts - result = %ErrorLevel%
  Loop, %workspaces_dir%\etb*.*, 0, 1                       ; Files only, recurse
    FileDelete, %A_LoopFileFullPath%

  IniDelete, %userIni%, D+DD, %appName%                     ; Enhanced Toolbars (ETB) FM-15en
  Finalise2()
ExitApp ; --- end MenuUnInstall

MenuReload:
  UpdateDocs()
  MsgUpdateOK()
ExitApp ; --- end MenuReload

; ============ functions ==========================================================================

CheckDownloadDoc(URL, fileTxt) {
  filestart := SubStr(fileTxt, 1, 8)
  if (filestart == "<!DOCTYP") {        ; Web error 404; PDF would start "%PDF-1.6" or the like
    MsgURLnotFound()
  }
}

ClearDesktop() { ; --- Clear Desktop -----------------------------------------------------------
  Sleep, 2000                                               ; assure last actions
  WinActivate, ahk_class Progman
  Send, {F5}                                                ; refresh desktop to make icon disappear
} ; --- end ClearDesktop

CreateShortCut() { ; --- Create shortcut to installation-pgm -----------------------------------
  If (!FileExist(shortCutDir)) {
    FileCreateDir, %shortCutDir%
  }
  FileCreateShortcut, %A_ScriptDir%\%appPgm%, %shortCutDir%%appStitle%.lnk, , , %appStitle%, %A_ScriptDir%\%appIcon%
  FileCreateShortcut, %A_ScriptDir%\ETB\%util1Name%.exe, %shortCutDir%%util1Title%.lnk, , , %util1Title%, %A_ScriptDir%\ETB\%util1Icon%
} ; --- end CreateShortCut

DownLoadDocs() { ; --- download the FM-version depnedent documentsa and replace place  holders
  WebSite := "https://daube.ch/docu/fm-documentation/FM/"
  WebFile := "fm" . FMTarget . "-UserGuide-" . FMlang . ".pdf"
  URLdownLoadToFile, %WebSite%%WebFile%, %installDir%\ETB-%FMlang%\fm-UserGuide.pdf
  FileReadLine, ResponseText, %installDir%\ETB-%FMlang%\fm-UserGuide.pdf, 1
  CheckDownloadDoc(WebSite . WebFile, ResponseText)

  WebFile := "fm" . FMTarget . "-INI-reference.pdf"
  URLdownLoadToFile, %WebSite%%WebFile%, %installDir%\ETB-%FMlang%\fm-INI-ref.pdf
  FileReadLine, ResponseText, %installDir%\ETB-%FMlang%\fm-INI-ref.pdf, 1
  CheckDownloadDoc(WebFile, ResponseText)

  WebFile := "fm" . FMTarget . "-MIF-reference.pdf"
  URLdownLoadToFile, %WebSite%%WebFile%, %installDir%\ETB-%FMlang%\fm-MIF-ref.pdf
  FileReadLine, ResponseText, %installDir%\ETB-%FMlang%\fm-MIF-ref.pdf, 1
  CheckDownloadDoc(WebFile, ResponseText)
} ; --- end DownLoadDocs

EditCfgFile() { ; --- edit CustomUI.cfg +appdata+ etc (no trailing slash!) ---------------------
  FMversion      := "FM-" . FMtarget
  ETBcustomuiFile   := "ETB-" . FMlang . "\etb-customui.cfg"
; MsgBox ETBcustomuiFile = %ETBcustomuiFile%
  findItems      := "+fm-root+|+appdata+|+vFM+|+FMtarget+|+vETB+|+date+|+help+|+etbdata+"
  replItems       = %FMroot%|%targetAppdata%|%FMversion%|%FMtarget%|%appVersion%|%appDate%|%helpFile%|%etbdata%
  aReplace       := StrSplit(replItems, "|")

  FileRead, customuiData, %ETBcustomuiFile%                 ; general file
  Loop, parse, findItems, |                                 ; for each item in the list
  {
    findItem := A_LoopField
    realText := aReplace[A_Index]
    realText := StrReplace(realText, "\", "/") ; Win => HTML notation for paths
    customuiData := StrReplace(customuiData, findItem, realText) ; Win => HTML notation
  }
  ETBmenuData := customuiData                               ; We need two different settings
; --- further preparation of the menu-data ----------------- See "Problems with ETB v11" in the Development-log
  findItems   := "+ETBflyout+|+ETBmenu+"
  if (fmTarget <= 15) {
    replItems := "«Add ETB_ToSentenceCap «Menu ETBChangeCaseMenu»» # inactive|<Add ETB_ToSentenceCap <Menu ETBChangeCaseMenu>> # active"
  } else {
    replItems := "<Add ETB_ToSentenceCap <Menu ETBChangeCaseMenu>> # active|<Add ETB_ToSentenceCap <Menu ETBChangeCaseMenu>> # active"
  }
  aReplace    := StrSplit(replItems, "|")
  Loop, parse, findItems, |                                 ; for each item in the list
  {
    findItem  := A_LoopField
    replItem  := aReplace[A_Index]
    ETBmenuData := StrReplace(ETBmenuData, findItem, replItem)
  }
; --- further preparation of the customuiData --------------
  findItem    := "+ETBflyout+"
  if (fmTarget <= 15) {
    replItem  := "<Add ETB_ToSentenceCap  <Menu ETBChangeCaseMenu>> # active"
  } else {
    replItem  := "«Add ETB_ToSentenceCap  «Menu ETBChangeCaseMenu»» # inactive"
  }
    customuiData := StrReplace(customuiData, findItem, replItem)
; --- final preparation of the customuiData (skip some parts) --------------
  searchRegex = (?s)\[SkipInCustomUI_cfg-Begin\].+\[SkipInCustomUI_cfg-End\]
  replace     = «-- Skipped in customui.cfg --»
  customuiData := RegExReplace(customuiData, searchRegex, replace, , 1)    ; repl once
; --- Avoid data in $HOME - use the alternative location in the installation dir
  customuiFile := "ETB-" . FMlang . "\customui.cfg"
  if (FileExist(customuiFile)) {
    FileDelete, %customuiFile%
  }
  FileAppend, %customuiData%, %customuiFile%
  customUILoc := A_ScriptDir . "\" . customuiFile
} ; --- end EditCfgFile

EditETBextras() { ; --- specify installation dir in script config file -------------------------
  scriptFIle := "scripts\ETBextras\ETBextras.cfg"
  if (FileExist(scriptFIle)) {
    IniWrite, %installDir%, %scriptFIle%, ETBinfo, ETBInstallation
  }
} ; --- end EditETBextras

EditIniFile() { ; --- Divert access to customUI.cfg, save version info -------------------------
; ConfigCustomUIFile=C:\Users\klaus\AppData\Roaming\D+DD\ETB14up\ETB-en\customui.cfg
  IniWrite, %customUILoc%, %userIni%, Files, ConfigCustomUIFile  ; Location of customUI.cfg
  IniWrite, %appVersion%, %userIni%, D+DD, %appName%        ; Enhanced Toolbars (ETB) FM-15en=v10.2
} ; --- end EditIniFile

EditMenu() { ; --- Append ETBmenuData to etb-menu --------------- FM 11 and up ----------------
; appFile = ETB\customui.cfg
  menuDirs = UnStructured\WYSIWYGView|Structured\WYSIWYGView
  Loop, parse, menuDirs, |                                  ; for each dir in the list
    {
    menuDir = %FMroot%\fminit\WorkSpaces\%A_LoopField%      ; read from $HOME
    inFile  = %menuDir%\menus\menus.cfg
    outFile = Workspaces-%FMlang%\%A_LoopField%\menus\etb-menus.cfg
    FileDelete, %outFile%                                   ; there may be an old one
    FileRead,   menu_ori, %inFile%                          ; FileCopy does not observe FileCoding
    FileAppend, %menu_ori%, %outFile%                       ; input may be Windows cp
    FileAppend, %ETBmenuData%, %outFile%                  ; append to etb-menu
    }
} ; --- end EditMenu

Finalise1() { ; --- Final steps at installation -------------------------------------------------
  RemoveInstFiles()
  MsgFinal()
} ; --- end Finalise1

Finalise2() { ; --- Final stepts at uninstallation ---------------------------------------------
  RemoveInstFiles()
  MsgUninstallOK()
} ; --- end Finalise2

GetCfgData() { ; --- Get data from cfg file ----------------------------------------------------
  IniRead, FMlang,     %cfgFile%, FrameMaker,   FMlang      ; en | de | fr given by FM installation
  IniRead, FMtarget,   %cfgFile%, FrameMaker,   FMtarget    ; 14

  IniRead, appLang,    %cfgFile%, Installation, appLang     ; defined by Windows
  IniRead, appName,    %cfgFile%, Installation, appName     ; ETB14up
  IniRead, appVersion, %cfgFile%, Installation, appVersion  ; 11.0
  IniRead, helpFile,   %cfgFile%, Installation, helpFile    ; fmaker75.html
  IniRead, installDir, %cfgFile%, Installation, installDir  ; %appdata%\D+DD\ETB14up-15

  IniRead, appVersion, %cfgFile%, Installation, appVersion
  IniRead, targetDir,  %cfgFile%, Installation, targetDir
  IniRead, auxDir,     %cfgFile%, Installation, auxDir
  IniRead, FMroot,     %cfgFile%, Installation, FMroot      ; Not terminated by \

  IniRead, appTitle,   %cfgFile%, %appLang%,    appTitle
  IniRead, appDescr,   %cfgFile%, %appLang%,    appDescr
  IniRead, preTitle,   %cfgFile%, %appLang%,    preTitle
  IniRead, preDescr,   %cfgFile%, %appLang%,    preDescr
  IniRead, instHint,   %cfgFile%, %appLang%,    instHint
  IniRead, option1,    %cfgFile%, %appLang%,    option1

  IniRead, btnContinue,%cfgFile%, %appLang%,    btnContinue
  IniRead, btnHelp,    %cfgFile%, %appLang%,    btnHelp
  IniRead, btnCancel,  %cfgFile%, %appLang%,    btnCancel
  IniRead, btnInstall, %cfgFile%, %appLang%,    btnInstall
  IniRead, btnUnInst,  %cfgFile%, %appLang%,    btnUnInst
  IniRead, btnDownload,%cfgFile%, %appLang%,    btnDownload
} ; // --- end GetCfgData

GetPdfModTime(pdfFilePath) { ; ___ Get the PDF modifytion file time -------------------------------
; https://www.pdflabs.com/tools/pdftk-the-pdf-toolkit/
 Static app := A_ScriptDir . "\pdftk.exe"                   ; PDF Tool Kit
      , out := A_MyDocuments . "\out.txt"                   ; temporary file
      , whichTime := "ModDate"                              ; CreationDate | ModDate
      , format := "yyyy-MM-dd HH:mm:ss"
 RunWait, "%app%" "%pdfFilePath%" dump_data output "%out%",, Hide
 FileRead, text, %out%
 FileRecycle, %out%
 RegExMatch(text, "s)" whichTime ".+?\K\d+", time)          ; \K pretends to start the match after it
 FormatTime, time, %time%, %format%
 Return time
} ; // --- end GetPdfModTime

RemoveInstFiles() { ; --- Remove Installation Preparation files -----------------------------------
; MsgBox,  auxDir = %auxDir%
  If (FileExist(auxDir . "\InstallThis")) {                 ; be aware of second install
    FileRemoveDir, %auxDir%\InstallThis, 1
    if ErrorLevel {
      MsgNoAccess()
      ExitApp
    }
    FileDelete, %auxDir%\PrepareInstallation.exe
    ClearDesktop()                                          ; assure removal of icons
  }
} ; --- end RemoveInstFiles

InstallSpecialDirs() { ; --- Workspace, icons --------------------------------------------------
; FileSetAttrib, -R, %targetAppdata%\toolbars\*.*, 1, 1     ; replacement not possible else ????

  FileCopyDir, WorkSpaces-%FMlang%, %workspaces_dir%, 1     ; dir and files
  FileCopyDir, etb_icons, %targetAppdata%\etb_icons, 1      ; dir and files
  FileCopyDir, scripts, %targetScripts%,  1                 ; dir and files
} ; --- end InstallSpecialDirs

UpdateDocs() { ; --- Replace local docs only if downloads are newer (PDF mod date)
  WebSite := "https://daube.ch/docu/fm-documentation/FM/"
  out := A_Temp                                             ; temporary folder
  UpdateOK = true

  WebFile     := "fm" . FMTarget . "-UserGuide-" . FMlang . ".pdf"
  TargetFile  := installDir . "\ETB-" . FMlang . "\fm-UserGuide.pdf"
  pdfFile = %installDir%\ETB-%FMlang%\fm-UserGuide.pdf
  dateCurrent := GetPdfModTime(pdfFile)
  URLdownLoadToFile, %WebSite%%WebFile%, %out%\fm-UserGuide.pdf
  pdfFile = %out%\fm-UserGuide.pdf
  dateNewer := GetPdfModTime(pdfFile)
  if (dateNewer > dateCurrent) {
    fileMove %out%\fm-UserGuide.pdf, %installDir%\ETB-%FMlang%\fm-UserGuide.pdf
  } else {
    MsgNoNewerFile("fm-UserGuide.pdf")
    UpdateOK = false
  }

  WebFile := "fm" . FMTarget . "-INI-reference.pdf"
  TargetFile  := installDir . "\ETB-" . FMlang . "\fm-INI-ref.pdf"
  pdfFile = %installDir%\ETB-%FMlang%\fm-INI-ref.pdf
  dateCurrent := GetPdfModTime(pdfFile)
  URLdownLoadToFile, %WebSite%%WebFile%,  %out%\fm-INI-ref.pdf
  pdfFile = %out%\fm-INI-ref.pdf
  dateNewer := GetPdfModTime(pdfFile)
  if (dateNewer > dateCurrent) {
    fileMove %out%\fm-INI-ref.pdf, %installDir%\ETB-%FMlang%\fm-INI-ref.pdf
  } else {
    MsgNoNewerFile("fm-INI-ref.pdf")
    UpdateOK = false
  }

  WebFile := "fm" . FMTarget . "-MIF-reference.pdf"
  TargetFile  := installDir . "\ETB-" . FMlang . "\fm-MIF-ref.pdf"
  pdfFile = %installDir%\ETB-%FMlang%\fm-MIF-ref.pdf
  dateCurrent := GetPdfModTime(pdfFile)
  URLdownLoadToFile, %WebSite%%WebFile%, %out%\fm-MIF-ref.pdf
  pdfFile = %out%\fm-INI-ref.pdf
  dateNewer := GetPdfModTime(pdfFile)
  if (dateNewer > dateCurrent) {
    fileMove %out%\fm-MIF-ref.pdf, %installDir%\ETB-%FMlang%\fm-MIF-ref.pdf
  } else {
    MsgNoNewerFile("fm-MIF-ref.pdf")
    UpdateOK = false
  }
;  FileRecycle, %out%                                       ; delete it
} ; --- end DownLoadDocs

; === Messages -===================================================================================
MsgFinal() {
  Progress, off
  if (appLang == "de" ) {
    MsgBox, 48, %appTitle%, Zusätzliche Menues`, Werkzeugleisten`, Skripte und Arbeitsbereiche für FM-%FMtarget% wurden installiert.`n%problem%
  } else if (FMlang == "fr" ) {
    MsgBox, 48, %appTitle%, Des menus`, barres d'outils`, scripts et espaces de travail supplémentaires ont été installés pour FM-%FMtarget%..`n%problem%
  } else {
    MsgBox, 48, %appTitle%, Additonal Menus`, Toolbars`, scripts and WorkSpaces for FM-%FMtarget% have been installed.`n%problem%
  }
} ; --- end MsgFinal

MsgUninstallOK()  {
  if (appLang == "de" ) {
    MsgBox, 48, %appTitle%, Enhanced Tool Bar (ETB) für FM-%FMtarget% wurde entfernt.`n-Das Installationsverzeichnis %A_ScriptDir% existiert noch.`n- Für eine Neuinstallation verwenden Sie einfach die Verknüpfung im Startmenü.`n- Für ein Update von ETB löschen Sie dieses Verzeichnis.`n`nUm das Ganze loszuwerden`, löschen Sie das Installationsverzeichnis %A_ScriptDir% und die Verknüpfung im Startmenü\D+DD\%targetDir%.`n`n%problem%
  } else if (FMlang == "fr" ) {
    MsgBox, 48, %appTitle%, La barre d'outils améliorée (ETB) pour FM-%FMtarget% a été supprimée.`nLe répertoire d'installation %A_ScriptDir% existe toujours.`n- Pour une réinstallation`, il suffit d'utiliser le raccourci dans le menu Démarrer.`n- Pour une mise à jour de l'ETB`, supprimez ce répertoire.`n`Pour vous débarrasser de tout`, supprimez le répertoire d'installation %A_ScriptDir% et le raccourci dans le menu Démarrer\D+DD\%targetDir%.`n`n%problem%
  } else {
    MsgBox, 48, %appTitle%, Enhanced Tool Bar (ETB) for FM-%FMtarget% has been removed.`nThe Installation directory %A_ScriptDir% still exists.`n• For re-Installation just use the short cut in the Start menu.`n• For an update of ETB delete this directory.`n`nTo get rid of the whole thing`, delete the Installation directory %A_ScriptDir% and the short cut in the Start menu\D+DD\%targetDir%.`n`n%problem%
  }
} ; --- end MsgUninstallOK

MsgATTRIBfailure() {
  if (appLang == "de" ) {
    MsgBox, 48, %appTitle%, Das Löschen von Dateien der Enhanced Toolbar ist fehlgeschlagen.`nEs ist nicht möglich`, %ErrorLevel%-Dateien im Verzeichnis %targetAppdata%\... zu löschen.`nDas Read-Only-Attribut kann nicht entfernt werden. Die Deinstallation wird für die anderen Dateien fortgesetzt.`n`nDie verbleibenden Dateien schaden Ihrem System nicht`, aber Sie können sie manuell mit dem Datei-Explorer unter %targetAppdata%\... entfernen.
    problem = Einige Dateien konnten nicht entfernt werden (Zugriffsprobleme). Möglicherweise möchten Sie sie manuell mit dem Datei-Explorer unter `n%targetAppdata% entfernen.
  } else if (FMlang == "fr" ) {
    MsgBox, 48, %appTitle%, La suppression des fichiers de la barre d'outils améliorée a échoué.`nIl n'est pas possible de supprimer les fichiers %ErrorLevel% dans le répertoire %targetAppdata%\...`nL'attribut Read-Only ne peut pas être supprimé. La désinstallation se poursuit pour les autres fichiers.`n`Les fichiers restants n'endommagent pas votre système`, mais vous pouvez les supprimer manuellement avec l'explorateur de fichiers dans %targetAppdata%\...
    problem = Certains fichiers n'ont pas pu être supprimés (problèmes d'accès)`nVous pouvez les supprimer manuellement avec l'explorateur de fichiers à l'adresse suivante : `n%targetAppdata%
  } else {
    MsgBox, 48, %appTitle%, Deleting files of the Enhanced Toolbar failed.`nIt is not possible to delete %ErrorLevel% files in the drectory %targetAppdata%\...`nThe Read-Only attribute can not be removed. De-Installation continues for the other files.`n`nThe left files do not harm Your system`, but You may wish to remove them manually with the File Explorer at %targetAppdata%\...
    problem = Some files could not be removed (access problems)`nYou may wish to remove them manually with the File Explorer at`n%targetAppdata%"
  }
} ; --- end MsgATTRIBfailure

MsgSomethingWrong() {
  if (appLang == "de" ) {
    MsgBox, 48, %appTitle%, Etwas ist schief gelaufen: A_ScriptDir <> `nScriptDir = %A_ScriptDir%`nInstallDir = %installDir%`n→ Höchstwahrscheinlich haben Sie nicht einfach das ZIP entpackt und`n- nur 2 Elemente (ein Programm`, ein Verzeichnis) erhalten,`n- sondern das ganze ZIP auf zahlreiche Dateien erweitert.`n`n→ Bitte entpacken Sie das ZIP auf den Desktop und doppelklicken Sie auf die exe-Datei.`nWenn das nicht erfolgreich ist`, geben Sie dem Programmierer bitte die Dateiinformationen von oben.
  } else if (FMlang == "fr" ) {
    MsgBox, 48, %appTitle%, Quelque chose s'est mal passé : A_ScriptDir <> `nScriptDir = %A_ScriptDir%`nInstallDir = %installDir%`n→ Il est fort probable que vous n'ayez pas simplement décompressé le ZIP et`n- obtenu seulement 2 éléments (un exe`, un répertoire),`n- mais que vous ayez étendu l'ensemble du ZIP à de nombreux fichiers.`n`n→ Veuillez décompresser le ZIP sur le bureau et double-cliquez sur le fichier exe.`nSi vous ne réussissez pas`, veuillez fournir au programmeur les informations sur le fichier ci-dessus..
  } else {
    MsgBox, 48, %appTitle%, Something went wrong: A_ScriptDir <> `nScriptDir  = %A_ScriptDir%`nInstallDir = %installDir%`n→ Most likely you did not simply unpack the ZIP and`n• got just 2 items (an exe`, a directory),`n• but expanded the whole ZIP to numerous files.`n`n→ Please unpack the ZIP to the desktop and double click the exe file.`nIf not successful`, please provide the programmer with the file information from above.
  }
} ; --- end MsgSomethingWrong

MsgNoAccess() {
  if (appLang == "de" ) {
    MsgBox, 48, %appTitle%, Die nicht mehr verwendeten Dateien aus der Installationsvorbereitung konnten nicht gelöscht werden (vermutlich Zugriffsbeschränkungen). Bitte lassen Sie Ihren Administrator diese Überbleibsel entfernen [sie schaden Ihrem System nicht`, es ist nur eine Bereinigung]..
  } else if (FMlang == "fr" ) {
    MsgBox, 48, %appTitle%, Les fichiers qui ne sont plus utilisés depuis la Préparation de l'installation n'ont pas pu être supprimés (probablement des restrictions d'accès).`n→ Veuillez laisser Votre administrateur supprimer ces restes [ils n'endommagent pas votre système`, c'est juste un nettoyage].
  } else {
    MsgBox, 48, %appTitle%, The no more used files from the Installation Preparation could not be deleted (propably access restrictions).`n→ Please let Your administrator remove these left overs [they do not harm your system`, it's just a cleaning up].
  }
} ; --- end MsgNoAccess

MsgNoNewerFile(fileName) {
  if (appLang == "de" ) {
    MsgBox, 48,  %appTitle%, Es gibt kein neueres %fileName% auf der Website des Autors. Das aktuelle wird behalten.
  } else if (FMlang == "fr" ) {
    MsgBox, 48,  %appTitle%, Il n'y a pas de %fileName% plus récent sur le site web de l'auteur. L'actuel est conservé.
  } else {
    MsgBox, 48,  %appTitle%, There is no newer %fileName% on the author's web site. Current one is kept.
  }
} ; --- end MsgNoNewerFile

MsgUpdateOK() {
  if (updateOK) {
    if (appLang == "de" ) {
      MsgBox, 48,  %appTitle%, Alle lokalen PDfs konnten durch neuere ersetzt werden.
    } else if (FMlang == "fr" ) {
      MsgBox, 48,  %appTitle%, Tous les PDF locaux peuvent être mis à jour.
    } else {
      MsgBox, 48,  %appTitle%, All local PDFs could be updated.
    }
  } else {
    if (appLang == "de" ) {
      MsgBox, 48,  %appTitle%, Nicht alle lokalen PDfs konnten durch neuere ersetzt werden.
    } else if (FMlang == "fr" ) {
      MsgBox, 48,  %appTitle%, Tous les PDF locaux n'ont pas pu être mis à jour.
    } else {
      MsgBox, 48,  %appTitle%, Not all local PDFs could be updated.
    }
  }
} ; --- end MsgNoNewerFile

MsgURLnotFound() {
  if (appLang == "de" ) {
    MsgBox, 48,  %appTitle%, URL "%URL%" konnte nicht gefunden werden (Server des Autors)`n`n Sie können den Autor unter klaus@daube.ch kontaktieren.
  } else if (FMlang == "fr" ) {
    MsgBox, 48,  %appTitle%, L'URL "%URL%" n'a pas pu être trouvée (Serveur de l'auteur)`n`n Vous pouvez contacter l'auteur à l'adresse klaus@daube.ch.
  } else {
    MsgBox, 48,  %appTitle%, URL «%URL%» could not be found (Server of the author)`n`n You may contact the author at klaus@daube.ch
  }
}