﻿/* ETBextras.jsx ====== UTF-8 =====================================================================
Documents     See ETB documentation ETB-13… (v9 and later)
Comment       • To be installed by ETB installation into Startup folder
              • Installation script must modify the menu files by adding
                <Order !MakerMainMenu.!FMidMain     <Before !MakerMainMenu.FileMenu>>
              • Localisation is not consistent. Some parts use the language defined at installation
                others use the FM-UI language. However, correct installation shoud provide no difference.

Attention     Code must be UTF-8. With Windows cp no umlauts etc. are allowed in menu items.
              In this script suite I use a universal object KLD_E (E for ETB)
              with functions and variables as properties to reduce the number of script ojects
              see E:\_DDDprojects\_JSX-library\TestOnly\DynTextObject.jsx
              → Functions must be defined before they are used!

Reference     Traduction française avec l'aide de https://www.deepl.com/translator par Klaus Daube
Copyleft      2019 Klaus Daube, Zürich, Switzerland, klaus@daube.ch
History       2019-11-26 FMident Renamed to ETBextras.jsx for integration of various functions
              2020-03-10 Released within ETB v9
              2022-09-27 Corrected: DocNav labels with control chars; View edit/print now observ session settings
              2022-09-27 Released within ETB v9.3
              2023-04-03 Updated ETBsaveAsOld
*/ ; // ===========================================================================================
#target framemaker

if (typeof KLD_E == "undefined") {
  KLD_E = {};                                     // Global object may already be defined by others
}

#include ETBextras\ETBinfo.jsx
#include ETBextras\ETBmultiCodeCommands.jsx
#include ETBextras\ETBsaveAsOld.jsx
#include ETBextras\ETBinsertSymbol.jsx
#include ETBextras\ETBdocNav.jsx
#include ETBextras\ETBrefresh.jsx

KLD_E.oInfo = {};                                 // holding the information about this installation
KLD_E.sExtrasCfg = "ETBextras\\ETBextras.cfg";    // Routines from this level read from the sub-dir
//KLD_E.sExtrasCfg = "ETBextras.cfg";             // If the reading routines would also be in sub-dir
KLD_E.ahkFile    = new File($.fileName.replace (/[^\\\/]+$/i , "ETBextras\\ETBextras.exe"));
// Console ("KLD_E.ahkFile: " +  KLD_E.ahkFile);

// >>>>>>>>>>>>>>> Main function must be at the end of the script with KLD_E = {} <<<<<<<<<<<<<<<<<

KLD_E.SetUpMenu = function (){ // === Create menu entry ===========================================
var bkFMidMenu, docFMidMenu, menuLocation, oMenus = {}, oCmd = {}, sFMversion;

  sFMversion = KLD_E.oInfo.FMversion.substring(0, 2);
  oMenus.MenuMain = sFMversion;                   // top menu can not execute a command
  oMenus.MenuInfo = "FM &Information";            // no localisation required
// ---------------------------------------------- Menu and Command definition for documents -------
  oCmd.MenuInfo = DefineCommand (1,"ETB_Info",  oMenus.MenuInfo, "");
  menuLocation  = app.GetNamedMenu ("!MakerMainMenu");
  docFMidMenu   = menuLocation.DefineAndAddMenu ("!ETB_Info", oMenus.MenuMain); // ! required for top menu
  docFMidMenu.AddCommandToMenu (oCmd.MenuInfo);

// No menu is defined here - command used in tool bar and context menus (sett up in customui.cfg / menus.cfg)
  oCmd.Doc = DefineCommand     (2, "ETB_InsertSymbol", "Insert special symbol [ETB]", "");

  menuLocation = app.GetNamedMenu ("!ETB_Info");   // Required, even if not used as menu - hence no localisation
  oCmd.DisplEdit = DefineCommand  (3, "ETB_EditingDisplay",  "Document Editing Display [ETB]",  "\\!qqe");
  oCmd.DisplPrnt = DefineCommand  (4, "ETB_PrintingDisplay", "Document Printing Display [ETB]", "\\!qqp");
  if (KLD_E.oInfo.UIlanguage == "de") {
    oCmd.SentenceC = DefineCommand  (7, "ETB_ToSentenceCap", "Satzanfang gross [ETB]", "");
  } else if (KLD_E.oInfo.UIlanguage == "fr") {
    oCmd.SentenceC = DefineCommand  (7, "ETB_ToSentenceCap", "Majuscules en début de phrase [ETB]", "");
  } else {                                        // default is English
    oCmd.SentenceC = DefineCommand  (7, "ETB_ToSentenceCap", "Sentence caps [ETB]", "");
  }
  oCmd.Refresh   = DefineCommand  (8, "ETB_Refresh", "Re-gain screen display [ETB]", "");

  menuLocation = app.GetNamedMenu ("FileMenu");
  oCmd.DocSaveOld = menuLocation.DefineAndAddCommand (5, "ETB_DocSaveOld", KLD_E.sDocMenu, "\\!qso");
  oCmd.DocSaveOld.KeyboardShortcutLabel = "ESC q s o";

  menuLocation  = app.GetNamedMenu ("ViewMenu");
  oCmd.DocNav = menuLocation.DefineAndAddCommand (6, "ETB_DocNav", localize (KLD_E.sDnav_title), "\\!qdn");
  oCmd.DocNav.KeyboardShortcutLabel = "ESC q d n";
// ---------------------------------------------- Menu and Command definition for books -----------
  oCmd.bkMenuInfo = DefineCommand (1,"ETB_InfoB",  oMenus.MenuInfo, "");;
  menuLocation = app.GetNamedMenu ("!BookMainMenu");
  bkFMidMenu   = menuLocation.DefineAndAddMenu ("!ETB_InfoB", oMenus.MenuMain); // ! required for top menu
  bkFMidMenu.AddCommandToMenu (oCmd.bkMenuInfo);

  menuLocation = app.GetNamedMenu ("BookFileMenu");
  oCmd.BookSaveOld = menuLocation.DefineAndAddCommand (5, "ETB_BookSaveOld", KLD_E.sBookMenu, "\\!qso");
  oCmd.BookSaveOld.KeyboardShortcutLabel = "ESC q s o";

  UpdateMenus();
} // --- end KLD_E.SetUpMenu

function Command(cmd) { // ==== assign function to command ========================================
/*            Assign functions to menu commands
Arguments     cmd            reference number of command from the menu definition
Returns       -
Called by     SetUpMenus, DebugMenu
Calling       GetInfo, PutInfo, DialogInsertSymbol, SetEditingDisplay, SetPrintingDisplay
              DialogSaveOld, DocNav, ToSentenceCap
Attention     This functions can only have this name (Command) and must use the classic syntax
              function Command (cmd) --- not --- Command = function (cmd) !
History       xxx
*/
	switch(cmd)  {	                                // Respond to menu command.
		case 1:
;     KLD_E.GetInfo ();
      KLD_E.PutInfo ();
      break;
		case 2:
      KLD_E.DialogInsertSymbol ();
      break;
		case 3:
      KLD_E.SetEditingDisplay ();
      break;
		case 4:
      KLD_E.SetPrintingDisplay ();
      break;
		case 5:
      KLD_E.DialogSaveOld ();
      break;
		case 6:
      KLD_E.DocNav ();
      break;
		case 7:
      KLD_E.ToSentenceCap();
      break;
		case 8:
      KLD_E.SetDisplay(true);
      KLD_E.RefreshDisplay();
      break;
    default:
  }
} // --- end Command

KLD_E.GetIniValue = function (sCfgFile, sValueName, vDefault) { // === Read an ini item into a variable
// Arguments  sCfgFile       ini file name (e.g. "
//            sValueName     is that used in the ini-file (e.g. "FMhome")
//            vDefault       default value, if not found in file
// Returns    Value from the ini file (string)
// Attention  This function does not observe ini sections. Hence the key names must be unique
// Comment    Empty lines, comments and section names are skipped
//            File errors reported only to Console - en only
// Used in    KLD_E.SetUpMenu
var match, notFound = true, regex, settingsFile, sFile, sFoundValue, thisLine;
    regex = {
      section: /^\s*\[\s*([^\]]*)\s*\]\s*$/,
      param: /^\s*([\w\.\-\_]+)\s*=\s*(.*?)\s*$/,
      comment: /^\s*;.*$/ } ;
  sFile = $.fileName.replace (/[^\\\/]+$/i , sCfgFile); // method Ric Quatro
  settingsFile = new File(sFile);
  settingsFile.open("r");
  if (settingsFile.error !== "") {
    Console ("Open write error for file " + sFile);
    return false;
  }

  while(!settingsFile.eof) {                      // Read and interprete lines of file
    thisLine = settingsFile.readln();
    if (thisLine === null || thisLine === "") {   // skip blank lines
      continue;
    }
    if (regex.comment.test(thisLine))  {          // skip comment lines
      continue;
    }
    if (regex.section.test(thisLine))  {          // ignore section lines
      continue;
    }
    match = thisLine.match(regex.param);          // ignore undefined contents
    if (match[1] == sValueName) {
      sFoundValue = match[2];
      notFound = false;
      break;                                      // no need to continue while loop
    }
  }
  settingsFile.close();
  if (notFound) {
    Console ("GetIniValue «" + sValueName + "» not found in configuration file:\n" + sCfgFile);
    return vDefault;
  }
//Console ("GetIniValue «" + sValueName + "» = «" + sFoundValue + "»");
  return sFoundValue;
} // --- end KLD_E.GetIniValue --------------------------------------------------------------------

KLD_E.PutIniValue = function (sCfgFile, sValueName, sValue) {
// Arguments  sCfgFile       ini file name (e.g. "
//            sValueName     key that is used in the ini-file (e.g. "FMhome")
//            sValue         value of the key
// Attention  This function does not observe ini sections. Hence the key names must be unique
//            If a key does not already exist, a message is issued and the key is added at the end.
// Comment    File errors reported only to Console - en only
// Returns    false on error
// Attention  Windows ini files are still only CP 1252, whareas maker.ini can be UTF-8 also
// Used in    KLD_E.DefineLevels
var aLines=[], aMatch, j, oRegEx, fSettings, nLines, sFile, notFound = true;

  sCfgFile = KLD_E.sExtrasCfg;
  oRegEx = {
    section: /^\s*\[\s*([^\]]*)\s*\]\s*$/,
    param: /^\s*([\w\.\-\_]+)\s*=\s*(.*?)\s*$/,   // only this used here
    comment: /^\s*;.*$/ } ;
  sFile = $.fileName.replace (/[^\\\/]+$/i , sCfgFile); // method Ric Quatro
  fSettings = new File(sFile); // method Ric Quatro
  fSettings.open("r");
  if (fSettings.error !== "") {
    Console ("Open read error for file " + sFile);
    return false;
  }
  while(!fSettings.eof) {                         // read entire file into array
    aLines.push(fSettings.readln());
  }
  fSettings.close();

  nLines = aLines.length;

  for (j = 0; j < nLines; j++ ) {                 // Does the key exist? --------------------------
    aMatch = aLines[j].match(oRegEx.param);
    if (aMatch == null) {continue;}
    if (aMatch[1].toUpperCase() == sValueName.toUpperCase()) {
      bNotFound = false;
      aLines[j] = sValueName + " = " + sValue;    // replace this line
      break;
    }
  }
  if (bNotFound) {                                // key does not exsit ---------------------------
    Console ("Value name " + sValueName + " not found in file «" + sCfgFile + "». Item added at the end");
    aLines.push (sValueName + " = " + sValue);
  }
  fSettings.open("w");                            // Write modified array -------------------------
  if (fSettings.error !== "") {
    Console ("Open write error for file " + sFile);
    return false;
  }
  nLines = aLines.length;
  for (j = 0; j < nLines; j++ ) {
    fSettings.writeln(aLines[j]);
  }
  fSettings.close();
  return true;
} // end of PutIniValue ---------------------------------------------------------------------------

KLD_E.GetFMlanguage = function () { // === Get the language of the FM installation ================
var FMlang;

  FMlang = app.Language;
	switch (FMlang) {
		case Constants.FV_LANG_ENGLISH:               // 0x01
		case Constants.FV_LANG_BRITISH: {             // 0x02
			KLD_E.FMlang = "en";
			break;
		}
		case Constants.FV_LANG_GERMAN:                // 0x03
		case Constants.FV_LANG_SWISS_GERMAN: {        // 0x04
			KLD_E.FMlang = "de";
			break;
		}
		case Constants.FV_LANG_FRENCH:                // 0x05
		case Constants.FV_LANG_CANADIAN_FRENCH: {     // 0x06
			KLD_E.FMlang = "fr";
			break;
		}
		default: {
			KLD_E.FMlang = "en";
		}
	}
	return;

} // end of GetFMlanguage -------------------------------------------------------------------------

// main function ==================================================================================
  KLD_E.GetFMlanguage();
  KLD_E.DocNavUIText(KLD_E.FMlang);
  KLD_E.InsertSymbolUIText(KLD_E.FMlang);
  KLD_E.MultiCodeCommandsUIText(KLD_E.FMlang);
  KLD_E.SaveAsOldUIText(KLD_E.FMlang);

  KLD_E.CreateInfo ();                            // define FM information in ETBextras.cfg
//Console ("main CreateInfo left");
  KLD_E.GetInfo ();                               // Get FM information
//Console ("main GetInfo left");
  KLD_E.GetSymbolDefs (KLD_E.FMlang);             // Get the symbol definitions
//Console ("main GetSymbolDefs left");

// --- switch between menu-invocation and ESTK debugging
  KLD_E.SetUpMenu ();                             // standard invocation
//Console ("main SetUpMenu left");
// KLD_E.DialogSaveOld();                         // direct call in debugging

