/* ETBextras.ahk ====== UTF-8 ========================================================================
              Set up contents in ETBextras.cfg

; Documentation E:\_DDDprojects\FM-toolbar00\Developer-log.fm
;               Traduction française avec l'aide de https://www.deepl.com/translator par Klaus Daube

Comment       • To be installed by ETB installation into Startup folder
              • Program is run at the start of FM by invocation from ETBextras.jsx
              • Some values in ETBextra.ini must be filled by the installation routines:
                ETBInstallation Set by Pre installation to config.ini
                InfoDate        Set here
                InfoFMhome      By Pre installation to config.ini
                InfoFMfull      Derived here from FMhome - it may change by FM update
                InfoFMedition   Derived from FMmarket (config.ini)
                InfoFMbitty     Can differ in FM-15
                InfoUIlanguage  From appLang (config.ini)
                InfoOSversion   May change (for W10) during the FM installation → to be set here

AHK version   1.1.29.00
              Assure that AutoHotkeySC.bin is the same as  Unicode 32-bit.bin
              Only this way the compiled scripts run on both x64 and x32 systems

Copyleft      2019 Klaus Daube, Zürich, Switzerland, klaus@daube.ch
History       2020-01-10 v1.0
*/ ; // ===========================================================================================

L0_SetUp:
  SetWorkingDir %A_ScriptDir%                               ; files are in subdirs
Global  installCfg
      , ETBInstallation                                     ; set in installation
      , extrasCfg                                           ; ETBextras.cfg
      , FMedition                                           ; FM-14 → 2017
      , FMfull                                              ; 15.0.4.751
      , FMlanguage                                          ; en | de | fr
      , FMmarket                                            ; <= cfg file 2019
      , FMroot                                              ; <= cfg file: NOT ending with \
      , FMtarget                                            ; 15
      , InfTime
      , OSversion                                           ; Win-7 32 bit

GetFMProps:
  FormatTime,  InfTime, , yyyy-MM-dd HH:mm                  ; current time

  extrasCfg := A_ScriptDir . "\ETBextras.cfg"
  IniRead, installCfg, %extrasCfg%, ETBinfo, ETBInstallation ; appdata\D+DD\FM15-toolbars-en (no change to BOM)
  installCfg := installCfg . "\config.cfg"

  IniRead, InfoFMhome, %installCfg%, Installation, FMroot   ; ends without \
  FMedition := SubStr(InfoFMhome, -3, 4)
  FileGetVersion, FMfull, %InfoFMhome%\FrameMaker.exe
  IniRead, FMlanguage, %installCfg%, FrameMaker, FMLang     ; both FM, ETB : en

  FMtarget  := SubStr(FMfull, 1 , 2)                        ; 15
  FMbitty   := FunFMbitty()

GetWinProps:
  If (A_Is64bitOS) {
    OSversion    := A_OSVersion . " 64 bit"
  } Else {
    OSversion    := A_OSVersion . " 32 bit"
  }

FillextrasCfg:
  IniWrite, %InfTime%,           %extrasCfg%, ETBinfo,  InfoDate
  IniWrite, %InfoFMhome%,        %extrasCfg%, ETBinfo,  InfoFMhome
  IniWrite, %FMfull%,            %extrasCfg%, ETBinfo,  InfoFMfull
  IniWrite, %FMedition%,         %extrasCfg%, ETBinfo,  InfoFMedition
  IniWrite, %FMbitty%,           %extrasCfg%, ETBinfo,  InfoFMbitty
  IniWrite, %FMlanguage%,        %extrasCfg%, ETBinfo,  InfoUIlanguage
  IniWrite, %OSversion%,         %extrasCfg%, ETBinfo,  InfoOSversion
  IniWrite, Unicode → arrow,     %extrasCfg%, ETBinfo,  UnicodeArrow ; does not destroy BOM

; MsgBox, Leaving ahk routine
exitapp

; ============ functions ==========================================================================
FunFMbitty() { ; --- check whether FM is installed as 32-bit or 64-bit version --------------------
; Returns    32 or 64
; Reference  https://forums.adobeprerelease.com/fm2018/discussion/comment/73#Comment_73
;           In the 64-bit version there is a "Legacy32" folder in FMHome, at least currently.
;           [Klaus Müller]
  If (A_Is64bitOS) {
    SetRegView, 64                      ; view registry as 64 bit app - AHK is 32 bit app
  }
	RegRead, FMHome, HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Adobe\FrameMaker\%FMtarget%, FMHome
  if (FMHome != "") {
;  MsgBox, FM 15 is installed as 32-bit version
  return 32
  }
	RegRead, FMHome, HKEY_LOCAL_MACHINE\SOFTWARE\Adobe\FrameMaker\%FMtarget%, FMHome
  if (FMHome != "") {
;  MsgBox, FM %FMtarget% is installed as 64-bit version
   return 64
  }
   return "??"
} ; --- end FunFMbitty

Messages:

