﻿/* ETBinsertSymbol.jsx ====== UTF-8 ===============================================================
              Insert special symbols, selectable from list - en version
Comment       Included by ETBextras.jsx
Reference     Based on SaveBookAsOldVersion 2013-09-04 by Carmen Publishing Inc. (www.frameexpert.com)
Attention     Code must be UTF-8. With Windows cp no umlauts etc. are allowed in menu items.
Copyleft      2017 Klaus Daube, Zürich, Switzerland, klaus@daube.ch
History       2017-00-30 v1.0 released:
              2018-01-16 v1.1 New command name with underscore
              2020-01-06 v2   become sub-function in ETBextra.jsx
              2022-11-09 The ES method for localisation uses the Windows language, not the FM one!
*/ ; // ===========================================================================================

KLD_E.aSymbolNames = [];                          // read in from sExtrasCfg

KLD_E.InsertSymbolUIText = function (FMlang) { // --- Set the UI texts for ETBinsertSymbol --------
  if (FMlang == "de") {
    KLD_E.sETBSymbols_title =  "Sonderzeichen einfügen [ETB]";
    KLD_E.sETBSymbols_NoDoc =  "Kein dokument aktiv";
    KLD_E.sETBSymbols_sInfo =  "Anstelle der Auswahl in der Liste kann auch direkt ein 2-hex-Zeichen FM Code (zb ae für ®) oder ein 4-hex-Zeichen Unicode (05d0 für א) eingegeben werden.\n\nDie Symbole Durchmesser and Math. Bruchstrich erfordern Schrift Segoe UI Symbol (oder anderes).";    KLD_E.sETBSymbols_sDouble= "Doppel-Klick für Auswahl";
    KLD_E.sETBSymbols_sSymbol= "Symbol";
    KLD_E.sETBSymbols_sInsert= "Symbol einfügen";
  } else if (FMlang == "fr") {
    KLD_E.sETBSymbols_title =  "Enserer symboles speciales [ETB]";
    KLD_E.sETBSymbols_NoDoc =  "Aucun document actif";
    KLD_E.sETBSymbols_sInfo =  "Au lieu de la sélection dans la liste, vous pouvez aussi entrer directement un code FM de 2 caractères hexadécimaux (par exemple ae pour ®) ou un Unicode de 4 caractères hexadécimaux (05d0 pour א).\n\nLes symboles de lignes fractionnaires Diameter et Math. nécessitent la police Segoe UI Symbol (ou autre).";
    KLD_E.sETBSymbols_sDouble= "Double clic pour la sélection";
    KLD_E.sETBSymbols_sSymbol= "Symbole";
    KLD_E.sETBSymbols_sInsert= "Insérer le symbole";
  } else {
    KLD_E.sETBSymbols_title =  "Insert Special Symbol [ETB]";
    KLD_E.sETBSymbols_NoDoc =  "No document active";
    KLD_E.sETBSymbols_sInfo =  "In place of selection from the list you may also directly enter a 2-hex-digits FM code (ae for ®) or a 4-hex-digits Unicode (05d0 for א).\n\nSymbols Diameter and Math fraction require font Segoe UI Symbol (or other).";
    KLD_E.sETBSymbols_sDouble= "Double click to select";
    KLD_E.sETBSymbols_sSymbol= "Symbol";
    KLD_E.sETBSymbols_sInsert= "Insert Symbol";
  }
} // --- end InsertSymbolUIText -------------------------------------------------------------------

KLD_E.GetSymbolDefs  = function (sLanguage) { // === Read the symbol defintions from CFG file =====
var j, jMax, sCfgFile= KLD_E.sExtrasCfg, sValueName;

  jMax = KLD_E.GetIniValue (sCfgFile, "nSymbols", 0);
//Console ("GetSymbolDefs " + sCfgFile + " " + jMax);
  for (j = 0; j < jMax; j++) {                    // 0 ...
    sValueName = sLanguage + "_" + (j + 1);
    KLD_E.aSymbolNames[j] = KLD_E.GetIniValue (sCfgFile, sValueName, undefined)
  }
//Console ("GetSymbolDefs " + KLD_E.aSymbolNames.length);
} // --- end KLD_E.GetSymbolDefs

KLD_E.DialogInsertSymbol = function (){ //=== Dialog to insert special symbols ====================
// Arguments -
// Returns   Inserts selected symbol into document text
// Called by main
// Calling   KLD_E.InsertSymbol, KLD_E.ToUnicode
// Reference ScriptUI for Dumies by kahrel@kahrel.plus.com (multicolumn lists)
var wDlg = new Window('dialog',KLD_E.sETBSymbols_title,undefined);
var j, nItems, iTab, sName, sItem

	wDlg.g1 = wDlg.add('group',undefined);

  	wDlg.g1.p1 = wDlg.g1.add('panel',undefined, KLD_E.sETBSymbols_sDouble);
    	wDlg.g1.p1.preferredSize.height = 250;

    	wDlg.g1.p1.SymbolNames = wDlg.g1.p1.add('listbox',undefined, undefined,{numberOfColumns: 2, showHeaders: false}); // to become two column list
    	wDlg.g1.p1.SymbolNames.graphics.font = ScriptUI.newFont ("dialog", "", 12);
    	wDlg.g1.p1.SymbolNames.size= [190,225];

  	wDlg.g1.p2 = wDlg.g1.add('panel',undefined,"Font Consolas, 48pt");
    	wDlg.g1.p2.preferredSize.height = 250;

    	wDlg.g1.p2.SymbolDisplay = wDlg.g1.p2.add('edittext',undefined,KLD_E.sETBSymbols_sSymbol);
    	wDlg.g1.p2.SymbolDisplay.graphics.font = ScriptUI.newFont ("Consolas", "", 48);

    	wDlg.g1.p2.Info = wDlg.g1.p2.add('statictext',undefined, KLD_E.sETBSymbols_sInfo, {multiline:true});
      wDlg.g1.p2.Info.graphics.font = ScriptUI.newFont ("dialog", "", 12);

    	wDlg.g1.p2.Insert = wDlg.g1.p2.add('button',undefined,KLD_E.sETBSymbols_sInsert);
// --- behaviour of edit field SymbolDisplay ------------------------------------------------------
  wDlg.g1.p2.SymbolDisplay.onChange = function () { // does this trigger if filled from list ?
    KLD_E.sETBSymbols_symbol = wDlg.g1.p2.SymbolDisplay.text;
  };
// --- fill the list with items and subitems ------------------------------------------------------
/*
DialogInsertSymbol number 58
Script Error :
Error Message      : undefined is not an object
Script, Line#   : C:\Users\klaus\AppData\Roaming\Adobe\FrameMaker\15\startup\ETBextras\ETBinsertSymbol.jsx,  81
*/
  nItems = KLD_E.aSymbolNames.length;
  for (j = 0; j < nItems; j++) {                  // make a 2-column list
//   Console (KLD_E.aSymbolNames[j]);
    iTab = KLD_E.aSymbolNames[j].indexOf("|");
    sName = KLD_E.aSymbolNames[j].substring (0, iTab);
    sItem = KLD_E.aSymbolNames[j].substring (iTab+1);
    with (wDlg.g1.p1.SymbolNames.add ("item", sName)) {
      subItems[0].text = sItem;
    }
  }
// --- behaviour of the lists ---------------------------------------------------------------------
// onChange would require an empty element (http://stackoverflow.com/questions/11877527)
// IMHO ondoubleClick is more user friendly as it indicates an action.
  wDlg.g1.p1.SymbolNames.onDoubleClick = function () {
  var listIndex, tmpText, strLen, sSymbol

    listIndex = wDlg.g1.p1.SymbolNames.selection.valueOf(),
    tmpText = KLD_E.aSymbolNames[listIndex];      // get the last character: the symbol
    strLen = tmpText.length;
    sSymbol = tmpText.substr (strLen-1, 1);
    if (/\d/.test(sSymbol)) {                     // notation of FM functional char
      sSymbol = tmpText.substr (strLen-2, 2);
    }
    wDlg.g1.p2.SymbolDisplay.text = sSymbol;
    KLD_E.sETBSymbols_symbol = sSymbol;
  };

// --- behaviour of the buttons -------------------------------------------------------------------
  wDlg.g1.p2.Insert.onClick = function () {
  var sCode, sSymbol, sUnicode, sAsci, ss

    sSymbol = KLD_E.sETBSymbols_symbol;
    if (sSymbol.length > 1) {                     // direct entry in edit field
      if (sSymbol.length == 2) {                  // this is a FramMaker function, e.g. 04 (optional hyphen)
        sAsci = String.fromCharCode(parseInt(sSymbol, 16));
        KLD_E.InsertSymbol (sAsci);
      }
      if (sSymbol.length == 4) {                  // this is a Unicode Char, e.g. 05D0 (א Alef)
        sUnicode = KLD_E.ToUnicode (sSymbol)
        KLD_E.InsertSymbol (sUnicode);
      }
    } else {                                      // selection from list
      KLD_E.InsertSymbol (sSymbol);
    }
    wDlg.close();
  }
  wDlg.show();
} // --- end KLD_E.DialogInsertSymbol

KLD_E.InsertSymbol  = function (sSymbol) { //=== Insert symbol at current text location ===========
// Arguments sSymbol:  the Unicode character to be inserted into the text
// Returns   -
// Called by DialogInsertSymbol
var oDoc, oTextRange

  oDoc = app.ActiveDoc;
  oTextRange = oDoc.TextSelection;
  oDoc.AddText(oTextRange.beg, sSymbol);          // insert the new text at the original beginning of the text range
  oTextRange.end.offset = oTextRange.beg.offset + 1; // 1character inserted
  newTextRange  = new TextRange(oTextRange.end, oTextRange.end);
  oDoc.TextSelection = newTextRange;              // place cursor behind inserted char
}

KLD_E.ToUnicode  = function (sCode) { //=== Convert unicode defintion to the character ============
// Arguments sCode: a string of the form hhhh with h being Hex figures
// Returns   Function is the resulting Unicode chacracter
// Called by DialogInsertSymbol
// Reference http://stackoverflow.com/questions/17267329/converting-unicode-character-to-string-format
var sCode, Unicode

  sUnicode = String.fromCharCode(parseInt(sCode,16))
  return sUnicode;
} // --- end KLD_E.InsertSymbol

