/* ETBmultiCodeCommands.jsx ====== UTF-8 ================================================================
              EditingDisplay and  PrintingDisplay (Substitute multi-code commands lost in FM-14)
Documentation www.daube.ch/docu/fmaker73.html
Comment       Included by ETBextras.jsx
Reference     Adobe: New Session-level scope for View menu commands: Earlier when you selected
              a command from the View menu, the command would impact only the active document.
              However, from 2017 release, if you choose any command from the View menu,
              it will impact all documents opened in the current session.
              The following commands in the View menu affect the structured and unstructured documents:
              • Borders  • Text Symbols  • Rulers  • Grid Lines  • Hotspot Indicators
              Traduction française avec l'aide de Babylon et Deepl par Klaus Daube
Copyleft      2017 Klaus Daube, Zürich, Switzerland, klaus@daube.ch
History       2017-06-30 v1.0 released:
              2018-01-16 v1.1 New command name with underscore
              2020-01-06 v2   become sub-function in ETBextra.jsx
              2022-11-09 The ES method for localisation uses the Windows language, not the FM one!
*/ ; // ===========================================================================================

KLD_E.asViewSettings = [];                        // save the user settings

KLD_E.MultiCodeCommandsUIText = function (FMlang) { // === Set the UI texts for ETBinsertSymbol ===
  if (FMlang == "de") {
    KLD_E.sDisplaying_NoDoc = "Kein dokument aktiv!";
  } else if (FMlang == "fr") {
    KLD_E.sDisplaying_NoDoc = "Aucun document actif !";
  } else {
    KLD_E.sDisplaying_NoDoc = "No document active!";
  }
} // --- end MultiCodeCommandsUIText --------------------------------------------------------------

KLD_E.SetEditingDisplay  = function () { // === Set view properties for Editing ===================
var oDoc,
    cmdBord = app.GetNamedCommand ("ViewBorders"),   // these are all toggles
    cmdTsym = app.GetNamedCommand ("ViewTextSymbols"),
    cmdRulr = app.GetNamedCommand ("ViewRulers"),
    cmdGrid = app.GetNamedCommand ("ViewGridLines"),
    cmdGraf = app.GetNamedCommand ("ToggleGraphicsDisplay");
    cmdBordOn  = app.GetNamedCommand ("ViewBordersOn"); // these are the session wide ones ?
    cmdBordOff = app.GetNamedCommand ("ViewBordersOff");
    cmdTsymOn  = app.GetNamedCommand ("ViewTextSymbolsOn");
    cmdTsymOff = app.GetNamedCommand ("ViewTextSymbolsOff");
    cmdRulrOn  = app.GetNamedCommand ("ViewRulersOn");
    cmdRulrOff = app.GetNamedCommand ("ViewRulersOff");
    cmdGridOn  = app.GetNamedCommand ("ViewGridLinesOn");
    cmdGridOff = app.GetNamedCommand ("ViewGridLinesOff");

if (!app.ActiveDoc.ObjectValid()) {
    alert ("ETBToggleViews: " + KLD_E.sDisplaying_NoDoc);
    return;
  }
  oDoc = app.ActiveDoc;
  if (oDoc.ViewBorders == 0) {
    Fcodes ([cmdBord.Fcode]);                     // => set Borders      ON
    Fcodes ([cmdBord.cmdBordOn]);
  }
  if (oDoc.ViewTextSymbols == 0) {
    Fcodes ([cmdTsym.Fcode]);                     // => set Textsymbols  ON
    Fcodes ([cmdTsymOn.Fcode]);
  }
//--- Leave rulers, grid and graphics as the user have set them before printing view
  if (oDoc.ViewRulers != KLD_E.asViewSettings [2]) {
    Fcodes ([cmdRulr.Fcode]);                     // => set Rulers
    Fcodes ([cmdRulrOn.Fcode]);
  }
  if (oDoc.ViewGrid != KLD_E.asViewSettings [3]) {
    Fcodes ([cmdGrid.Fcode])                      // => set Gridlines
    Fcodes ([cmdGridOn.Fcode])
  }
  if (oDoc.ViewNoGraphics != KLD_E.asViewSettings [4]) {
    Fcodes ([cmdGraf.Fcode])                      // => set Graphics
  }
  return;
} // --- end SetEditingDisplay

KLD_E.SetPrintingDisplay  = function () { // === Set view properties for Printing =================
var oDoc,
    cmdBord = app.GetNamedCommand ("ViewBorders"),       // these are all toggles
    cmdTsym = app.GetNamedCommand ("ViewTextSymbols"),
    cmdRulr = app.GetNamedCommand ("ViewRulers"),
    cmdGrid = app.GetNamedCommand ("ViewGridLines"),
    cmdGraf = app.GetNamedCommand ("ToggleGraphicsDisplay");
    cmdBordOn  = app.GetNamedCommand ("ViewBordersOn"); // these are the session wide ones ?
    cmdBordOff = app.GetNamedCommand ("ViewBordersOff");
    cmdTsymOn  = app.GetNamedCommand ("ViewTextSymbolsOn");
    cmdTsymOff = app.GetNamedCommand ("ViewTextSymbolsOff");
    cmdRulrOn  = app.GetNamedCommand ("ViewRulersOn");
    cmdRulrOff = app.GetNamedCommand ("ViewRulersOff");
    cmdGridOn  = app.GetNamedCommand ("ViewGridLinesOn");
    cmdGridOff = app.GetNamedCommand ("ViewGridLinesOff");

  if (!app.ActiveDoc.ObjectValid()) {
    alert ("ETBToggleViews: " + KLD_E.sDisplaying_NoDoc);
    return;
  }
  oDoc = app.ActiveDoc;
  KLD_E.asViewSettings [0] = oDoc.ViewBorders;    // save previous view
  KLD_E.asViewSettings [1] = oDoc.ViewTextSymbols;
  KLD_E.asViewSettings [2] = oDoc.ViewRulers;
  KLD_E.asViewSettings [3] = oDoc.ViewGrid;
  KLD_E.asViewSettings [4] = oDoc.ViewNoGraphics;
  if (oDoc.ViewBorders == 1) {
    Fcodes ([cmdBord.Fcode]);                     // => set Borders      OFF
    Fcodes ([cmdBordOff.Fcode]);
  }
  if (oDoc.ViewTextSymbols == 1) {
    Fcodes ([cmdTsym.Fcode]);                     // => set Textsymbols  OFF
    Fcodes ([cmdTsymOff.Fcode]);
  }
  if (oDoc.ViewRulers == 1) {
    Fcodes ([cmdRulr.Fcode])                      // => set Rulers       OFF
    Fcodes ([cmdRulrOff.Fcode])
  }
  if (oDoc.ViewGrid == 1) {
    Fcodes ([cmdGrid.Fcode])                      // => set Gridlines    OFF
    Fcodes ([cmdGridOff.Fcode])
  }
  if (oDoc.ViewNoGraphics == 1) {
    Fcodes ([cmdGraf.Fcode])                      // => set Graphics    On mandatory
  }
  return;
} // --- end SetPrintingDisplay

KLD_E.ToSentenceCap = function() {
// Returns    The modified text is no more selected (same as with the commands from cmds.cfg)
// Called by  ETB-scripts > ETBmultCodeCommands
// Calling    GetText
// Attention  This function ruins any character formats in the selection.
//            It is probably necessary to apply the Uppercasing to the individual text items.
//            But currently (2020-03-31) i do not see how...
var cmdBord, oDoc = app.ActiveDoc, oTR = oDoc.TextSelection, sFMversion;
var aMatch = [], string, regEx = /[.!?] *\w/g, sChar1;

  sFMversion = KLD_E.oInfo.FMversion.substring(0, 2);
  if (sFMversion > 15) {                          // FM-16 provides this function naturally
    cmdBord = app.GetNamedCommand ("SetToSentenceCap"); // \x425
    Fcodes ([cmdBord.Fcode]);                        // perform command
    return
  }
  if (oTR.end.offset == 0) {
    oTR.end.obj = oTR.beg.obj;
    oTR.end.offset = Constants.FV_OBJ_END_OFFSET - 1;
    app.ActiveDoc.TextSelection = oTR;
  }
  string = KLD_E.GetText(oTR,oDoc);

  aMatch = string.match (regEx);
  if (aMatch != null) {
    nMatch = aMatch.length;
    for (j = 0; j < nMatch; j++) {                // convert those found
      string = string.replace (aMatch[j], aMatch[j].toLocaleUpperCase ());
    }
  } else {                                        // only arbitrary text
    string = string + " ";                        // prophylactically add blank
  }
  sChar1 = string.substr(0,1).toLocaleUpperCase(); // initial char in selection
  string = string.replace(string.substr(0,1), sChar1);
  oDoc.Clear(0);                                  // clear selection
  oDoc.AddText(oTR.beg, string);                  // insert the new text at original place
//oTR.end.offset = string.length-1;               // leave out to be consistent with behaviour
//app.ActiveDoc.TextSelection = oTR;              // of commands from cmds.cfg - see Developer-log
  return
} //--- end ETBToSentenceCap

KLD_E.GetText = function (oText, oDoc) { //=== Get the sText from the oText object ================
// Arguments oText: object to get the string from
//           oDoc:  current document
// Returns   Function value =  the text string
// Called by xxx
// Reference Rick Quatro's function Get-text-from-any-object
// Used in   FMbiblio ? FMcalc
var aTextItems, iTxtLen, j, sText= "";

  if (oText.constructor.name !== "TextRange") {   // Get a list of the strings in the text object or TR
    aTextItems = oText.GetText(Constants.FTI_String);
  } else {
    aTextItems = oDoc.GetTextForRange(oText, Constants.FTI_String);
  }
  iTxtLen = aTextItems.len;
  for (j = 0; j < iTxtLen; j += 1) {              // Concatenate the strings
    sText += (aTextItems[j].sdata);
  }
  return sText;                                   // Return the sText
} //--- end GetText
