/* ETBRefresh.jsx ====== UTF-8 ====================================================================
              Regain display.
Documentation www.daube.ch/docu/fmaker73.html
Comment       Included by ETBextras.jsx
              Misbehaving scripts may leave the display hidden.
              Use this function to re-gain screen display.
              Traduction française avec l'aide de Deepl par Klaus Daube
Copyleft      2021 Klaus Daube, Zürich, Switzerland, klaus@daube.ch
History       2021-05-27  become sub-function in ETBextra.jsx
*/ ; // ===========================================================================================

KLD_E.SetDisplay = function (bWanted) { // ========================================================
/*
Arguments     bWanted: true => set display ON; false => display OFF
Called by     various
Atention      Switching visibility to often lets this function fail.
              See https://forums.adobe.com/thread/2276988
Reference     Rick Quattro
Used in       FMbiblio, FMcalc, FMvars
*/
  if (app.Displaying != bWanted) {
    app.Displaying = bWanted;
  }
} //--- end SetDisplay ----------------------------------------------------------------------------

KLD_E.RefreshDisplay = function () { // ===========================================================
/*            Refresh display, equivalent to CTRL+l
Called by     FMbiblioResolve, FMbiblioExpand
Comment       The following does nothing at all:
                FcodeList = new Array(FCodes.KBD_REFRESHWIN);
                Fcodes(FcodeList);
              The following 'official functions' are not accepted:
                app.ActiveDoc.ReDisplay(); // app.ActiveDoc.ReDisplay is not a function
                app.ActiveDoc.ReFormat();  // app.ActiveDoc.ReFormat is not a function
              Any action on the (hidden) master page does the trick.
TestEnv       FM-14, ES-toolkit CC (4.0.0.1) ES 4.5.5
Calling       -
*/
  app.ActiveDoc.Redisplay();                      // Redisplay, not ReDisplay
/* this works also - but is overkill
  oDoc = app.ActiveDoc
  oMasterPage = oDoc.FirstMasterPageInDoc;
  oTextFrame = oDoc.NewTextFrame (oMasterPage.PageFrame);
  oTextFrame.Delete ();
*/
} // --- end RefreshDisplay -----------------------------------------------------------------------
