﻿/* ETBSaveAsOld.jsx ====== UTF-8 ==================================================================
              Save FM document or book as old version
Documentation www.daube.ch/docu/fmaker73.html
Comment       To be placed in Startup folder of user area
              For a list of DDD used Esc shortcuts see E:\_DDDprojects\FM-GUI-modifications\ESC-sequences\FM-ESC-sequences.fm
Reference     Based on SaveBookAsOldVersion 2013-09-04 by Carmen Publishing Inc. (www.frameexpert.com)
              Traduction française avec l'aide de Babylon et Deepl par Klaus Daube
Attention     Code must be UTF-8. With Windows cp no umlauts etc. are allowed in menu items.
Copyleft      2017 Klaus Daube, Zürich, Switzerland, klaus@daube.ch
History       2016-05-24 v1.0 Klaus Daube, Docu+Design Daube, klaus@daube.ch
              2016-06-03 v1.1 Save as MIF added.
                         It is not possible to save to another MIF version as the currnt FM
                         or to MIF-7 (FV_SaveFmtInterchange70)
                         Only Constants.FV_SaveFmtInterchange is valid in our case
              2017-04-25 v1.2 Adapted for FM-14 and integration into ETB
              2017-06-30 v1.3 Revert to dialogue, Inform user about current document location (for ordinary save)
              2018-01-16 v1.4 New command names with underscore
              2018-04-20 v1.5 Checkbox for overwriting current with warning
              2018-06-29 v1.6 Hint about Rick Quatro's script
              2020-01-09 v1.7 become sub-function in ETBextra.jsx
              2022-11-09 v1.8 The ES method for localisation uses the Windows language, not the FM one!
              2023-03-27 v1.9 Added support for FM-16, 17
                               It is not possible to save MIF as specific as the FM version
                               (► save creates FV_BadSaveScriptValue in return parms)
*/ ; // ===========================================================================================

KLD_E.sSavO_AppVersion = "v1.9";              // version of this script

KLD_E.SaveAsOldUIText = function () { // === Set the UI texts for ETBinsertSymbol ===========
  if (KLD_E.FMlang == "de") {
    KLD_E.sSavO_title     = "Buch oder Dokument in älterer Version speichern";
    KLD_E.sDocMenu        = "Dokument in älterer Version speichern …";
    KLD_E.sBookMenu       = "Buch in älterer Version speichern …";
    KLD_E.sSavO_NoDoc     = "Kein Buch oder Dokument aktiv";
    KLD_E.sSavO_Target    = "Zielverzeichnis ist notwendig";
    KLD_E.sSavO_NoTarget  = "Zielverzeichnis nicht vorhanden";
    KLD_E.sSavO_SelTarget = "Bitte zielverzeichnis auswählen";
    KLD_E.sSavO_UnknownFM = "Aktuelle FM version ist nicht ≥10";
    KLD_E.sSavO_Warning   = "Dies überschreibt die aktuelle Version! Wollen Sie das wirklich?";
    KLD_E.sSavO_Success   = "Dokument oder Buch gespeichert\n\nAchtung: Das aktuelle verzeichnis ist immer noch das Zielverzeichnis!";
  } else if (KLD_E.FMlang == "fr") {
    KLD_E.sSavO_title     = "Enregistrer le livre ou document en version plus ancienne";
    KLD_E.sDocMenu        = "Enregistrer document comme version ancienne …";
    KLD_E.sBookMenu       = "Enregistrer livre comme version ancienne …";
    KLD_E.sSavO_NoDoc     = "Aucun livre ou document actif";
    KLD_E.sSavO_Target    = "Dossier cible est requise";
    KLD_E.sSavO_NoTarget  = "Le dossier cible n'existe pas";
    KLD_E.sSavO_SelTarget = "Veuillez sélectionner un dossier ciblee";
    KLD_E.sSavO_UnknownFM = "Version FM actuelle n'est pas ≥ 10";
    KLD_E.sSavO_Warning   = "Ceci remplace la version actuelle ! C'est ce que tu veux vraiment ?";
    KLD_E.sSavO_Success   = "Document ou livre enregistré\n\nAttention : Notez que le répertoire courant est toujours le dossier cible !";
  } else {
    KLD_E.sSavO_title     = "Save book or document in older version";
    KLD_E.sDocMenu        = "Save document as old version …";
    KLD_E.sBookMenu       = "Save book as old version";
    KLD_E.sSavO_NoDoc     = "No book or document active";
    KLD_E.sSavO_Target    = "Target folder is required.";
    KLD_E.sSavO_NoTarget  = "Target folder does not exist";
    KLD_E.sSavO_SelTarget = "Please select a target folder";
    KLD_E.sSavO_UnknownFM = "Current FM version is not ≥10";
    KLD_E.sSavO_Warning   = "This will overwrite the current version! Do your really want this?";
    KLD_E.sSavO_Success   = "Document or book saved\n\nAttention: Be aware that the current directory is still the Target Folder!";
  }
} // --- end SaveAsOld ----------------------------------------------------------------------------

/*
KLD_E.oSavO_ActiveBook, KLD_E.oSavO_ActiveDoc;
KLD_E.sSavO_FMversion;                            // FM-8 ... FM-15
KLD_E.iETBsavO_FileType;                          // Constants.FV_SaveFmtBinary... | ...FV_SaveFmtInterchange
*/

KLD_E.DialogSaveOld = function () { // ============================================================
var stVersion, stFolder, stAdvice, sbBrowse, sbSaveFM, sbSaveMIF, scCurrent, sbHelp, sbCancel,
    sbClose, stHint;

  if (!app.ActiveBook.ObjectValid() && !app.ActiveDoc.ObjectValid()) {
    alert ("ETBSaveAsOld: " + KLD_E.sSavO_NoDoc);
    return;
  }
  if (KLD_E.FMlang == "de") {
    stVersion = "FM Zielversion";
    stFolder  = "Zielverzeichnis";
    stAdvice  = "Verzeichnis eingeben oder Schaltfläche Durchsuchen benutzen";
    sbBrowse  = "Durchsuchen";
    sbSaveFM  = "Als FM speichern";
    sbSaveMIF = "Als MIF speichern";
    scCurrent = "An den aktuellen Ort speichern ";
    sbHelp    =  "Hilfe";
    sbCancel  = "Abbrechen";
    sbClose   = "OK, schliessen";
    stHint    = "Dieses Skript speichert die konvertierten Dateien in nur einem Verzeichnis.\nMIF wird immer in derselben Version wie der aktuelle FrameMaker gespeichert - nicht in der ausgewählten Zielversion.\nRick Quatro's ConvertDocs speichert die konvertierten Dateien im entsprechenden Quellverzeichnis. Dieses Skript im StartUp Verzeichnis kann gegen Rick's ausgetauscht werden. Siehe www.framexpert.com/store ";
  } else if (KLD_E.FMlang == "fr") {
    stVersion = "FM version cible";
    stFolder  = "Dossier cible";
    stAdvice  = "Entrez le nom du dossier ou utilisez le bouton Parcourir";
    sbBrowse  = "Parcourir";
    sbSaveFM  = "Enrgistrer comme FM";
    sbSaveMIF = "Enrgistrer comme  MIF";
    scCurrent = "Enregistrer à l'emplacement actuel";
    sbHelp    = "Aide";
    sbCancel  = "Annuller";
    sbClose   = "OK, conclure";
    stHint    = "Ce script enregistre tous les fichiers dans le même répertoire.\nLa MIF est toujours sauvegardée dans la même version que la FM actuelle - et non dans la version sélectionnée pour la cible.\nLe script ConvertDocs de Rick Quatro enregistre les fichiers convertis dans le même répertoire que la source. Vous pouvez remplacer ce script dans le dossier StrartUp par Rick's. Voir www.framexpert.com/store ";
  } else {
    stVersion = "FM target version";
    stFolder  = "Target folder";
    stAdvice  = "Enter folder name or use Browse button";
    sbBrowse  = "Browse";
    sbSaveFM  = "Save as FM";
    sbSaveMIF = "Save as MIF";
    scCurrent = "Save to current location";
    sbHelp    = "Help";
    sbCancel  = "Cancel";
    sbClose   = "OK, close";
    stHint    = "This script saves all files to the same directory.\nMIF is always saved in the same version as the current FM - not in the version selected for the target.\nRick Quatro's ConvertDocs script saves the converted files to the same directory as the source comes from. You may replace this script in the StrartUp folder by Rick's. See www.frameexpert.com/store ";
  }

  wSaveOld = new Window('dialog', KLD_E.sSavO_title + "  " + KLD_E.sSavO_AppVersion, undefined);
  wSaveOld.size = [670, 210];
  wSaveOld.spacing = 0;
  wSaveOld.margins = 0;
  wSaveOld.alignChildren = "left";
  wSaveOld.sp1 = wSaveOld.add("statictext", undefined, "");                   // space
	wSaveOld.g1 = wSaveOld.add('group',undefined);
  wSaveOld.g1.spacing = 5;
  wSaveOld.g1.margins = 0;
  wSaveOld.g1.indent = 5;
  	wSaveOld.g1.st1 = wSaveOld.g1.add('statictext',undefined,stVersion);
  	wSaveOld.g1.rb = wSaveOld.g1.add('group',undefined);
    	wSaveOld.g1.rb.rb8  = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-8");
    	wSaveOld.g1.rb.rb9  = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-9");
    	wSaveOld.g1.rb.rb10 = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-10");
    	wSaveOld.g1.rb.rb11 = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-11");
    	wSaveOld.g1.rb.rb12 = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-12");
    	wSaveOld.g1.rb.rb13 = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-13");
    	wSaveOld.g1.rb.rb14 = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-14");
    	wSaveOld.g1.rb.rb15 = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-15");
    	wSaveOld.g1.rb.rb16 = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-16");
    	wSaveOld.g1.rb.rb17 = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-17");
     wSaveOld.rbtxt = wSaveOld.add('statictext',undefined,"[2015]         [2017]        [2019]         [2020]        [2022]");
     wSaveOld.rbtxt.indent = 390
  wSaveOld.sp2 = wSaveOld.add("statictext", undefined, "");                   // space
	wSaveOld.g2 = wSaveOld.add('group',undefined);
  wSaveOld.g2.indent = 5;
  	wSaveOld.g2.st2        = wSaveOld.g2.add('statictext',undefined,stFolder);
  	wSaveOld.g2.folderName = wSaveOld.g2.add('edittext',undefined,stAdvice);
  	wSaveOld.g2.folderName.preferredSize.width = 310;;
  	wSaveOld.g2.bBrowse    = wSaveOld.g2.add('button',undefined,sbBrowse);
  wSaveOld.sp3 = wSaveOld.add("statictext", undefined, "");                   // space
	wSaveOld.g3 = wSaveOld.add('group',undefined);
  wSaveOld.g3.indent = 5;
  	wSaveOld.g3.alignChildren = ['left',' '];
  	wSaveOld.g3.cCurrent = wSaveOld.g3.add("checkbox", undefined,scCurrent);
  	wSaveOld.g3.bSaveFM  = wSaveOld.g3.add('button',undefined,sbSaveFM);
  	wSaveOld.g3.bSaveMIF = wSaveOld.g3.add('button',undefined,sbSaveMIF);
  	wSaveOld.g3.bHelp    = wSaveOld.g3.add('button',undefined,sbHelp);
  wSaveOld.sp4 = wSaveOld.add("statictext", undefined, "");                   // space
	wSaveOld.g4 = wSaveOld.add('group',undefined);
  wSaveOld.g4.indent = 5;
  	wSaveOld.g4.alignChildren = ['left',' '];
  	wSaveOld.g4.st3        = wSaveOld.g4.add('statictext',undefined,localize(stHint),{multiline:true});
  	wSaveOld.g4.st3.preferredSize = {width:500, height:60};   // at least 4 lines
	  wSaveOld.g4.st3.graphics.foregroundColor = wSaveOld.g4.st3.graphics.newPen (wSaveOld.g4.st3.graphics.PenType.SOLID_COLOR, [1, 0, 0], 1);
	wSaveOld.g5 = wSaveOld.add('group',undefined);
  wSaveOld.g5.indent = 5;
	wSaveOld.g5.alignChildren = ['left',' '];
  	wSaveOld.g5.Close    = wSaveOld.g5.add('button',undefined,sbClose);
  	wSaveOld.g5.Cancel   = wSaveOld.g5.add('button',undefined,sbCancel);
// --- set the radio buttons --- depending on the current FM version ------------------------------
  var appVersion = app.VersionMajor;
  var rbuttons   = wSaveOld.g1.rb;
  for (var j = 0; j < rbuttons.children.length; j++) {
    var txt = rbuttons.children[j].text;
    rbuttons.children[j].value = false;
    rbuttons.children[j].visible = true;
    if (j+8 == appVersion ) {
      rbuttons.children[j].value = true;
    }
    if (j+8 > appVersion ) {
//    rbuttons.children[j].visible = false;
      rbuttons.children[j].enabled = false;  // make them inactive
    }
  }
// --- behaviour of edit field
  wSaveOld.g2.folderName.onChange = function () {
    gsFolder = wSaveOld.g2.folderName.text;
    wSaveOld.g4.st3.text = "-----";               // remove the intial hint message
 }
// --- behaviour of checkbox
  wSaveOld.g3.cCurrent.onClick = function () {
    if (wSaveOld.g3.cCurrent.value) {
      path = KLD_E.GetCurrentFolder ();
      wSaveOld.g2.folderName.text = path;
      wSaveOld.g4.st3.text = KLD_E.sSavO_Warning;
    } else {
      wSaveOld.g2.folderName.text = stAdvice;
    wSaveOld.g4.st3.text = "-----";
    }
//  wSaveOld.show();
  }
// --- behaviour of the buttons
  wSaveOld.g2.bBrowse.onClick  = KLD_E.ButtonBrowse;
  wSaveOld.g3.bSaveFM.onClick  = KLD_E.ButtonSaveFM;
  wSaveOld.g3.bSaveMIF.onClick = KLD_E.ButtonSaveMIF;
  wSaveOld.g3.bHelp.onClick    = KLD_E.ButtonHelp;
  wSaveOld.g5.Close.onClick    = function () {
    wSaveOld.close();
  }
  wSaveOld.g5.Cancel.onClick   = function () {
    wSaveOld.close();
  }
// --- final action in dialgue
	wSaveOld.g5.Close.enabled = false;
	wSaveOld.g5.Cancel.enabled = true;
  wSaveOld.show();
} // --- end DialogSaveOld

KLD_E.ButtonBrowse  = function () { //=============================================================
  var userFolder = new Folder(app.OpenDir);
  var targetFolder = userFolder.selectDlg(KLD_E.sSavO_SelTarget);
  if (targetFolder !== null) {
    wSaveOld.g2.folderName.text = targetFolder.fsName;
  }
} // --- end KLD_E.ButtonBrowse

KLD_E.ButtonSaveFM  = function () { //=============================================================
//$.bp(true);
  KLD_E.bETBsavO_MIF = false;
  KLD_E.ButtonSave ();
} // --- end KLD_E.ButtonSaveFM

KLD_E.ButtonSaveMIF  = function () { //============================================================
//$.bp(true);
  KLD_E.bETBsavO_MIF = true;
  KLD_E.ButtonSave ();
} // --- end KLD_E.ButtonSaveMIF

KLD_E.ButtonSave  = function () { // ==============================================================
  var folderText = wSaveOld.g2.folderName.text;
  var lastBSlash, fileType;
    if (folderText === "") {                      // Validate the target folder.
    alert (KLD_E.sSavO_Target);
    return;
  } else {                                        // Make sure the folder exists.
    if (Folder(folderText).exists === false) {
      alert (KLD_E.sSavO_NoTarget);
      return;
    } else {
      lastBSlash = folderText.lastIndexOf("\\");
      if (lastBSlash == folderText.length-1) {    // remove terminating \
        folderText = folderText.substring(0, lastBSlash);
      }
      wSaveOld.g2.folderName.text = folderText;
      KLD_E.oSavO_Folder = folderText;            // target folder
    }
  }
  KLD_E.sSavO_FMversion = KLD_E.SelectedRadioButton (wSaveOld.g1.rb); // text from selected radio button
  KLD_E.iETBsavO_FileType = KLD_E.GetFileType (KLD_E.sSavO_FMversion);
// --- for a book
  if (app.ActiveBook.ObjectValid()) {
    KLD_E.oSavO_ActiveBook = app.ActiveBook;
    KLD_E.ProcessBook (KLD_E.oSavO_ActiveBook.Name);
  }
// --- for single document
  if (app.ActiveDoc.ObjectValid()) {
    KLD_E.oSavO_ActiveDoc = app.ActiveDoc;
    KLD_E.ProcessDoc (KLD_E.oSavO_ActiveDoc.Name);
  }
  KLD_E.UpdateStatus (KLD_E.sSavO_Success);
} // --- end KLD_E.ButtonSave

KLD_E.ButtonHelp  = function () {
//$.bp(true);
  KLD_E.OpenWebURL ("http://www.daube.ch/docu/files/ETB14up-UG-" + KLD_E.FMlang + ".pdf");
} // --- end KLD_E.ButtonHelp

KLD_E.SelectedRadioButton  = function (rbuttons) { //==============================================
  for (var j = 0; j < rbuttons.children.length; j++) {
    if (rbuttons.children[j].value == true){
      return rbuttons.children[j].text;
    }
  }
} // --- end KLD_E.SelectedRadioButton

KLD_E.UpdateStatus  = function (sStatus) { // =====================================================
  wSaveOld.g5.Close.enabled = true;
  wSaveOld.g5.Cancel.enabled = false;
  wSaveOld.g4.st3.text = sStatus;
} // --- end KLD_E.UpdateStatus

KLD_E.GetCurrentFolder  = function () { // ========================================================
  oDoc = app.ActiveDoc;
  if (oDoc.ObjectValid()) {
    path = oDoc.Name;
  } else {
    oBook = app.ActiveBook;
      if (oDoc.ObjectValid()) {
      path = oBook.Name;
    }
  }
  lastBSlash = path.lastIndexOf("\\");            // \ needs escaping
  folder = path.substring(0, lastBSlash);
  return folder
} // --- end KLD_E.GetCurrentFolder

KLD_E.ProcessDoc  = function (name) { // ==========================================================
  var oDoc;
  oDoc = KLD_E.GetDocOrBook (name);               // Open the document.
  if (oDoc.obj) {
    KLD_E.SaveAsFileType (oDoc.obj, KLD_E.oSavO_Folder, KLD_E.iETBsavO_FileType); // Save and close the document.
    if (oDoc.isOpen === false) {
      oDoc.obj.Close (true);
    }
  }
} // --- end KLD_E.ProcessDoc

KLD_E.ProcessBook  = function (name) { // =========================================================
  var book = KLD_E.GetDocOrBook (name);           // Open the book.
  var bookComp = book.obj.FirstComponentInBook;
  while (bookComp.ObjectValid()) {
    if (bookComp.BookComponentFileType === Constants.FV_BK_FM) { // Document
      KLD_E.ProcessDoc (bookComp.Name);
    }
    bookComp = bookComp.NextBookComponentInDFSOrder;
  }
// update the book (component paths); save the book and close it.
  KLD_E.UpdateComponentPaths (book.obj, KLD_E.oSavO_Folder, KLD_E.iETBsavO_FileType);
  KLD_E.SaveAsFileType (book.obj, KLD_E.oSavO_Folder, KLD_E.iETBsavO_FileType);
  if (book.isOpen === false) {
    book.obj.Close (true);
  }
} // --- end KLD_E.ProcessBook

KLD_E.ProcessComponents  = function (book) { // ===================================================
  var bookComp = book.FirstComponentInBook;
  while (bookComp.ObjectValid()) {
    if (bookComp.BookComponentFileType === Constants.FV_BK_FM) { // Document
      KLD_E.ProcessDoc (bookComp.Name);
    }
    bookComp = bookComp.NextBookComponentInDFSOrder;
  }
} // --- end KLD_E.ProcessComponents

KLD_E.GetFileType  = function (fmVersion) { // ====================================================
// Attention    List must correspoind to UI radio buttons wSaveOld.g1.rb
  if (KLD_E.bETBsavO_MIF) {
        KLD_E.iETBsavO_FileType =  Constants.FV_SaveFmtInterchange;
  } else {
    switch (fmVersion) {
      case "FM-8":
        KLD_E.iETBsavO_FileType =  Constants.FV_SaveFmtBinary80; break;
      case "FM-9":
        KLD_E.iETBsavO_FileType = Constants.FV_SaveFmtBinary90; break;
      case "FM-10":
        KLD_E.iETBsavO_FileType = Constants.FV_SaveFmtBinary100; break;
      case "FM-11":
        KLD_E.iETBsavO_FileType = Constants.FV_SaveFmtBinary110; break;
      case "FM-12":
        KLD_E.iETBsavO_FileType = Constants.FV_SaveFmtBinary120; break;
      case "FM-13":
        KLD_E.iETBsavO_FileType = Constants.FV_SaveFmtBinary130; break;
      case "FM-14":
        KLD_E.iETBsavO_FileType = Constants.FV_SaveFmtBinary140; break;
      case "FM-15":
        KLD_E.iETBsavO_FileType = Constants.FV_SaveFmtBinary150; break;
      case "FM-16":
        KLD_E.iETBsavO_FileType = Constants.FV_SaveFmtBinary160; break;
      case "FM-17":
        KLD_E.iETBsavO_FileType = Constants.FV_SaveFmtBinary170; break;
      default:
        KLD_E.iETBsavO_FileType = KLD_E.GetDefaultFileType();
    }
  }
  return KLD_E.iETBsavO_FileType;                 // return also in function
} // --- end KLD_E.GetFileType

KLD_E.GetDefaultFileType  = function () { // ======================================================
// Comment    Only FM 10 upwards are relevant (start of script support)
  var appVersion = app.VersionMajor;
  if (KLD_E.bETBsavO_MIF) {
        return Constants.FV_SaveFmtInterchange;
  } else {
    switch (appVersion) {
      case "10":
        KLD_E.iETBsavO_FileType = Constants.FV_SaveFmtBinary100; break;
      case "11":
        KLD_E.iETBsavO_FileType = Constants.FV_SaveFmtBinary110; break;
      case "12":
        KLD_E.iETBsavO_FileType = Constants.FV_SaveFmtBinary120; break;
      case "13":
        KLD_E.iETBsavO_FileType = Constants.FV_SaveFmtBinary130; break;
      case "14":
        KLD_E.iETBsavO_FileType = Constants.FV_SaveFmtBinary140; break;
      case "15":
        KLD_E.iETBsavO_FileType = Constants.FV_SaveFmtBinary150; break;
      case "FM-16":
        KLD_E.iETBsavO_FileType = Constants.FV_SaveFmtBinary160; break;
      case "FM-17":
        KLD_E.iETBsavO_FileType = Constants.FV_SaveFmtBinary170; break;
      default:
        alert (KLD_E.sSavO_UnknownFM);
        exit;
    }
  }
} // --- end KLD_E.GetDefaultFileType

KLD_E.OpenDocOrBook  = function (filename) { // ===================================================
  var i = 0, doc = 0;
  var docOrBook = {obj: 0, isOpen: false};        // Make an object to return.
  var openProps = GetOpenDefaultParams();         // Get default property list for opening documents or books.
  var retParm = new PropVals();                   // Get a property list to return any error messages.
                                                  // Set specific open property values to open the document or book.
  i=GetPropIndex(openProps,Constants.FS_AlertUserAboutFailure);
  openProps[i].propVal.ival=false;
  i=GetPropIndex(openProps,Constants.FS_MakeVisible);
  openProps[i].propVal.ival=false;
  i=GetPropIndex(openProps,Constants.FS_FileIsOldVersion);
  openProps[i].propVal.ival=Constants.FV_DoOK;
  i=GetPropIndex(openProps,Constants.FS_FileIsInUse);
  openProps[i].propVal.ival=Constants.FV_ResetLockAndContinue;
  i=GetPropIndex(openProps,Constants.FS_FontChangedMetric);
  openProps[i].propVal.ival=Constants.FV_DoOK;
  i=GetPropIndex(openProps,Constants.FS_FontNotFoundInCatalog);
  openProps[i].propVal.ival=Constants.FV_DoOK;
  i=GetPropIndex(openProps,Constants.FS_FontNotFoundInDoc);
  openProps[i].propVal.ival=Constants.FV_DoOK;
  i=GetPropIndex(openProps,Constants.FS_RefFileNotFound);
  openProps[i].propVal.ival=Constants.FV_AllowAllRefFilesUnFindable;

  doc = Open(filename,openProps,retParm);         // Attempt to open the document or book.
  if (doc.ObjectValid() === 1) {
      docOrBook.obj = doc;
  } else {                                        // If the document can't be open, print the errors to the Console.
      PrintOpenStatus(retParm);
  }
  return docOrBook;                               // Return the document or book object.
} // --- end KLD_E.OpenDocOrBook

KLD_E.GetDocOrBook  = function (filename) { // ====================================================
  if (/\.fm$/i.test (filename) === true) {        // See if the document or book is already open.
    var docOrBook = KLD_E.DocIsOpen (filename);         // Doc
  } else {                                        // Book
    var docOrBook = KLD_E.BookIsOpen (filename);
  }

  if (docOrBook.obj) {
    return docOrBook;
  } else {                                        // The document is not already open, so open it.
    return KLD_E.OpenDocOrBook (filename);
  }
} // --- end KLD_E.GetDocOrBook

KLD_E.DocIsOpen  = function (filename) { // =======================================================
  var file = File(filename);                      // Make a File object from the file name.
  var fileName = file.fullName.toUpperCase();     // Uppercase the filename for easy comparison.
  var doc = {obj: 0, isOpen: false};              // Make an object to return.
  var currDoc = app.FirstOpenDoc;                 // Loop through the open documents in the session.
  while (currDoc.id) {              // Compare the document’s name with the one we are looking for.
    if (File(currDoc.Name).fullName.toUpperCase() === fileName) { // The document we are looking for is open.
      doc.obj = currDoc;                          // Add it to the object and return it.
      doc.isOpen = true;
      return doc;
    }
    currDoc = currDoc.NextOpenDocInSession;
  }
  return doc;                                     // Document is not open; return the “empty” object.
} // --- end KLD_E.DocIsOpen

KLD_E.BookIsOpen  = function (filename) { // ======================================================
  var file = File(filename);                      // Make a File object from the file name.
  var fileName = file.fullName.toUpperCase();     // Uppercase the filename for easy comparison.
  var book = {obj: 0, isOpen: false};             // Make an object to return.
  var currBook = app.FirstOpenBook;               // Loop through the open books in the session.

  while (currBook.id) {             // Compare the document’s name with the one we are looking for.
    if (File(currBook.Name).fullName.toUpperCase() === fileName) { // The document we are looking for is open.
      book.obj = currBook;                        // Add it to the object and return it.
      book.isOpen = true;
      return book;
    }
    currBook = currBook.NextOpenBookInSession;
  }
  return book;                                    // Book is not open; return the “empty” object.
} // --- end KLD_E.BookIsOpen

KLD_E.UpdateComponentPaths  = function (book, folder, fileType) { // ==============================
  var bookComp = book.FirstComponentInBook;

  while (bookComp.ObjectValid()) {
    if (bookComp.BookComponentFileType === Constants.FV_BK_FM) { // Document
      bookComp.Name = folder +  /\\[^\\]+$/.exec (bookComp.Name)[0];
//  } else if (bookComp.BookComponentFileType === Constants.FV_BK_BOOK) {
//    bookComp.Name = folder +  /\\[^\\]+$/.exec (bookComp.Name)[0];
    }
    bookComp = bookComp.NextBookComponentInDFSOrder;
  }
  KLD_E.SaveAsFileType (book, folder, fileType);
} // --- end KLD_E.UpdateComponentPaths

KLD_E.SaveAsFileType  = function (docOrBook, folder, fileType) { // ===============================
  var params, returnParams, k, fullName, baseName, lastBSlash, lastDot, savePath, extension;
  params = GetSaveDefaultParams();
  returnParams = new PropVals();
  k = GetPropIndex (params, Constants.FS_FileType);
  params[k].propVal.ival = fileType;
  fullName = docOrBook.Name;
  lastDot    = fullName.lastIndexOf( "." ) ;      // .fm or .book
  lastBSlash = fullName.lastIndexOf("\\");
  baseName = fullName.substr(lastBSlash+1, lastDot-lastBSlash-1);
  extension  = fullName.substr(lastDot);          // .book or .fm
  if (KLD_E.bETBsavO_MIF) {extension = ".mif";}
  savePath = folder + "\\" + baseName + extension;
  docOrBook.Save (savePath, params, returnParams);
	if (FA_errno !== 0) {
		PrintSaveStatus (returnParams);
	}
//$.bp(true);
} // --- end KLD_E.SaveAsFileType

KLD_E.OpenWebURL  = function (URL) { // ===========================================================
  var path = KLD_E.GetScriptPath () + "\\";
// Create an executable (batch, shortcut) file and execute it
  var tempFile = File (path + "WebShortCut.url");
  tempFile.open("w");
  tempFile.encoding = "UTF-8";
  tempFile.writeln ("[{000214A0-0000-0000-C000-000000000046}]");
  tempFile.writeln ("Prop3=19,2");
  tempFile.writeln ("[InternetShortcut]");
  tempFile.writeln ("URL=" + URL);
  tempFile.close();
  tempFile.execute ();
  tempFile.delete;
} // --- end KLD_E.OpenWebURL

KLD_E.GetScriptPath  = function () { //============================================================
// Returns path in the function
  var scrPath, winPath, lastBSlash;

// $.fileName: script path (including the file-name of the script)
// platform independent format e.g. /e/_DDDprojects/FM+EN-escript/escript
  scrPath = $.fileName;
  winPath = new File (scrPath);
  scrPath = winPath.fsName;                       // get Windows format
  lastBSlash = scrPath.lastIndexOf("\\");         // \ needs escaping
  scrPath = scrPath.substring(0, lastBSlash);
  return scrPath;                                 // no final \ !
} // --- end KLD_E.GetScriptPath

/*
function SwitchUILanguage () {; // === Switch language according to application language ==========
  switch (app.Language) {
    case Constants.FV_LANG_BRITISH:
    case Constants.FV_LANG_ENGLISH: {
      $.locale = "en"; break;
    }
    case Constants.FV_LANG_CANADIAN_FRENCH:
    case Constants.FV_LANG_FRENCH: {
      $.locale = "fr"; break;
    }
    case Constants.FV_LANG_GERMAN:
    case Constants.FV_LANG_NEW_GERMAN:
    case Constants.FV_LANG_NEW_SWISS_GERMAN:
    case Constants.FV_LANG_SWISS_GERMAN:	{
      $.locale = "de"; break;
    }
    default: {
      $.locale = "en"; break;
    }
  }
} // --- end SwitchUILanguage
*/